/**
 * ConceptCodesplits is in configure-require.js as ConceptCodesplitsRequireJS.
 * This module has the same exported functions as ConceptModulePromisesWebpack.
 * ConceptCodesplits gets overridden to be ConceptCodesplitsWebpack in webpack.concept2.js for Concept v2.
 * Inside of webpack, we will use the webpack version of the functions below.
 *  If you ever add a function here, you need to add it to ConceptModulePromisesWebpack too
 * (when we remove requireJS from the concept page completely, this file can be removed!)
 *
 * Concept v1 (requireJS) we end up getting ConceptCodesplitsRequireJS and in Concept v2 (webpack)
 * we get ConceptCodesplitsWebpack. They each have functions with the same names and arguments,
 * and they all return a promise that resolves to the module. In the webpack version, it
 * gets loaded as a dynamic code split, and in requirejs it gets loaded as a normal RequireJS require.
 *
 * This file split addresses the fact that we need a different syntax in Webpack than in RequireJS
 * to get dynamic code splits. In Webpack we use the import(foo) syntax from the ECMAScript proposal
 * for dynamic imports, which RequireJS doesn't support:
 * https://webpack.js.org/guides/code-splitting/#dynamic-imports
 *
 * In time we'll likely add a handful of other code splits.
 *
 * When the regression of webpack is done and we're using webpack in the original concept page,
 * we can get rid of the ConceptCodesplitsRequireJs.
 */
define(function(require) {
  const {getRequireJsPromise} = require('common/js/app/WebpackUtils')

  return {
    getTeachingElementPromise: (teName, teVersion) => {
      const tePath = `te/${teName}/${teVersion}/courseware/main`;
      return getRequireJsPromise(tePath);
    },
    getTourFactoryPromise: (moduleRequirePath = 'tours/js/TourFactory') => getRequireJsPromise(moduleRequirePath),

    /**
     * Add two code splits, one for kaltura low quality, one for kaltura rtmp.
     * These code splits are used in tep-jsl/0.1/components/kaltura3rdPartyCode/main.coffee
     * to load either HQ or LQ kaltura player on the page, respectively (depending on the wave settings).
     * Kaltura promises.
     * @param moduleRequirePath - the RequireJS module to load
     * @returns Promise<Module>
     */
    getKalturaLQPromise: (moduleRequirePath = 'kaltura_low_quality') => getRequireJsPromise(moduleRequirePath),
    getKalturaHQPromise: (moduleRequirePath = 'kaltura_rtmp') => getRequireJsPromise(moduleRequirePath),
  };

});
