define([
    "jquery",
    "underscore",
    "backbone",
    'backbone.radio',
    "marionette",
    "bootstrap",
    "sidebar/SidebarRegion",
    "text!sidebar/SidebarViewTemplate.html",
    "tep/tep"
], function(
    $,
    _,
    Backbone,
    BackboneRadio,
    Marionette,
    bootstrap,
    SidebarRegion,
    template,
    TEP
) {

    const searchIsEnabledOnAPlatformLevel = window.globals.searchEnabled;

    // Accessibility enhancements
    $(document).keyup(function(e){
        if (e.keyCode == 13) {
            // If enter key is clicked then open sidebar menu
            if ($(e.target).hasClass("search-li-sidebar") && searchIsEnabledOnAPlatformLevel){
                $('#search').trigger('enter');
            } else if ($(e.target).hasClass("peerhelp-li-sidebar")){
                $('#peerHelp').trigger('enter');
            } else if ($(e.target).hasClass("td-li-sidebar")){
                $('#teamDiscussion').trigger('enter');
            } else if ($(e.target).hasClass("utility-note-add")){
                $('#tools_sticky_note').trigger('enter');
            } else if ($(e.target).hasClass("utility-colors-pick")){
                $('#tools_color').trigger('enter');
            } else if ($(e.target).hasClass("utility-begin-draw")){
                $('#draw').trigger('enter');
            } else if ($(e.target).hasClass("bookmark-li-sidebar")){
                $('#tools_bookmark').trigger('enter');
            } else if ($(e.target).hasClass("settings-li-sidebar")){
                $('#settings').trigger('enter');
            } else if ($(e.target).hasClass("dashboard-li-sidebar")){
                location.href = $('#course_dashboard_url').attr("href");
            }
        }
    });

    // Views
    var SidebarView = Marionette.LayoutView.extend({
        template: _.template(template),
        el: "#conceptSidebar",
        events: {
            "click #syllabus": "toggleSyllabus",
            "enter #syllabus": "toggleSyllabus",
            "click #search": "toggleSearch",
            "enter #search": "toggleSearch",
            "click #draw": "toggleDraw",
            "enter #draw": "toggleDraw",
            "click #drawChoice > li": "beginDraw",
            "click #tools_color": "colorMenu",
            "enter #tools_color": "colorMenu",
            "click #color_choice > li": "colorMenuElements",
            "click #color_choice > input": "colorMenuElements",
            "mousedown #color_choice>input:radio + label": "colorPicker",
            "click #tools_bookmark": "bookmarkConcept",
            "enter #tools_bookmark": "bookmarkConcept",
            "click #tools_sticky_note": "addStickyNote",
            "enter #tools_sticky_note": "addStickyNote",
            "click #stop_editing": "stopEditing",
            "click #peerHelp": "togglePeerHelp",
            "enter #peerHelp": "togglePeerHelp",
            "click #glossary": "toggleGlossary",
            "click #teamDiscussion": "toggleTeamDiscussion",
            "enter #teamDiscussion": "toggleTeamDiscussion",
            "click #settings": "toggleSettings",
            "enter #settings": "toggleSettings",
            "click .expand-utility": "expandView",
            "click .sidebar-item > li": "sendMetric"
        },
        drawingModes: {
            edit_white: {status: "edit", line: undefined, element: "info_edit_draw"},
            brushLrg_white: {status: true, line: 15, element: "info_draw"},
            brushMed_white: {status: true, line: 5, element: "info_draw"},
            brushSml_white: {status: true, line: 1, element: "info_draw"}
        },
        student_tools: null,
        drawingsEmpty: '{"objects":[],"background":""}',
        regionClass: SidebarRegion,
        regions: {
            utilityPane: ".utility-pane"
        },
        dataOptions: {
            userId: window.user_id,
            is_watcher: window.globals.is_watcher,
            is_invisible: window.globals.is_invisible,
            courseId: window.course_instance,
            conceptId: window.globals.concept,
            cohortId: window.cohort_id,
            waveId: window.wave_id,
            glossary_url: window.glossary_url,
            glossary_ready: window.glossary_ready,
            iNeedHelpEnabled: window.iNeedHelpEnabled,
            drawings: window.tools_drawings,
            notes: window.tools_notes,
            hide_team_discussion: window.hide_team_discussion
        },
        sidebarTypes: {
            syllabus: "syllabus",
            search: "search",
            peerHelp: "peerHelp",
            glossary: "glossary",
            teamDiscussion: "teamDiscussion",
            settings: "settings"
        },
        sidebarTypeToggleMethods: {
            syllabus: "toggleSyllabus",
            search: "toggleSearch",
            peerHelp: "togglePeerHelp",
            glossary: "toggleGlossary",
            teamDiscussion: "toggleTeamDiscussion",
            settings: "toggleSettings"

        },
        // List of push notification types that sidebar needs to handle
        sidebarNotificationTypes: ["peerHelpQuestion", "peerHelpComment", "teamDiscussion"],
        disabledTypes: [],
        pos: 0,
        expandStatus: false,
        expandSpeed: 600,
        expandWidthMin: 349,
        expandWidthMax: 699,
        expandOpacityMin: 0.4,
        expandOpacityMax: 1,
        conceptBody: $("#conceptBody"),
        teachingElementContainer: $('#sticky-container'),
        titlesContainer: $('.concept-page-titles'),
        body: $("body"),
        expandAnimateOpElements: null,
        lastView: {view: null, type: null},
        currentTeachingElement: null,
        tepReady: false,
        menuItemElementClass: "sidebar-menu-item",
        menuItemCurrentClass: "current",

        initialize: function(){
            this.region = this.getRegion("utilityPane");
            this.initRouter();
            this.listenTEP();
            this.metricsChannel = BackboneRadio.channel("metrics");
            this.disabledComponentsChannel = BackboneRadio.channel("disabledComponents");

            const self = this;
            this.disabledComponentsChannel.on('disabledComponentsUpdated', function(disabledComponentsList) {
                self.disabledComponentsCheck(disabledComponentsList);
            })
        },

        onRender: function() {
            this.setBookmark(window.bookmarked);
            if(!Backbone.History.started) Backbone.history.start();
            this.$el.find(".tip").tooltip();
            this.$el.find("#dashboard_link").attr('href', window.user_dashboard_url);
            if(window.user_course_dashboard_url)
                this.$el.find("#course_dashboard_url").attr('href', window.user_course_dashboard_url);
            else
                this.$el.find("#course-dashboard-menu-item").hide();
            this.expandAnimateOpElements = this.$el.find(".utility-begin-draw, .utility-colors-pick, .utility-note-add");
        },

        templateHelpers: function() {
            return {
                search_enabled: searchIsEnabledOnAPlatformLevel,
                glossary_enabled: this.viewEnabled(this.sidebarTypes.glossary),
                peerhelp_enabled: this.viewEnabled(this.sidebarTypes.peerHelp),
                teamdiscussion_enabled: this.viewEnabled(this.sidebarTypes.teamDiscussion),
                hide_team_discussion: this.dataOptions.hide_team_discussion
            }
        },

        initRouter: function() {
            // Map routes with controller methods
            // IMPORTANT: Keep route names the same as sidebarTypes keys for route tracking reasons
            var appRouteHandler = {
                syllabus: "sidebarSyllabus",
                search: "sidebarSearch",
                peerHelp: "sidebarPeerHelp",
                glossary: "sidebarGlossary",
                teamDiscussion: "sidebarTeamDiscussion",
                settings: "sidebarSettings",
                "te/:id": "positionToTei"
            };

            // Controller which contains the methods which are used as route handlers
            var _this = this;
            this.appRouterController = {
                sidebarSyllabus: function() {
                    _this.toggleSyllabus();
                },
                sidebarSearch: function() {
                    _this.toggleSearch();
                },
                sidebarPeerHelp: function() {
                    _this.togglePeerHelp();
                },
                sidebarGlossary: function() {
                    _this.toggleGlossary();
                },
                sidebarTeamDiscussion: function() {
                    _this.toggleTeamDiscussion();
                },
                sidebarSettings: function() {
                    _this.toggleSettings();
                },
                positionToTei: function(id) {
                    _this.currentTeachingElement = id;
                    _this.positionToTei();
                }
            };

            // Define an AppRouter constructor
            // Create a new instance of the AppRouter, assign the routes and controller
            var router = Marionette.AppRouter.extend({});
            this.appRouter = new router({
                appRoutes: appRouteHandler,
                controller: _this.appRouterController
            });
        },

        checkLastView: function(type) {
            // Check was previous view of the same type and if so run closing animation
            var status = this.lastView.type == type;
            if (status)
                this.onCloseAnimation();
            return status;
        },

        /**
         * this.lastView contains an object with two keys: view and type.
         *
         * * `view` is an instance of MarionetteView for the last sidebar component
         * that was activated.
         * * `type` is the string corresponding to the name of the sidebar menu item,
         * for example, "syllabus".
         *
         * This method updates this.lastView when the view changes. If a menu item
         * closes, it fires the 'before:close' triggerMethod on that view.
         * If a menu item opens, it fires the 'before:open' triggerMethod on that view.
         * @param view - Marionette view
         * @param type - string corresponding to the name of the sidebar view activated
         */
        setLastView: function(view, type) {

            if (this.lastView.type !== type && this.lastView.view) {
                this.lastView.view.triggerMethod('before:close');
            }

            if (view) {
                view.triggerMethod('before:open');
            }

            this.lastView = {view: view, type: type};
        },

        viewEnabled: function(type) {
            // Checks required to see if a view is supposed to be disabled.
            var viewDisabledConditions = [
                this.disabledTypes.indexOf(type) != -1,
                !this.dataOptions.iNeedHelpEnabled && type == this.sidebarTypes.peerHelp,
                !this.dataOptions.glossary_ready && type == this.sidebarTypes.glossary
            ];
            // If View Disabled Conditions contain one true use case that means this view should be disabled.
            if (viewDisabledConditions.indexOf(true) != -1)
                return false;
            // In any other case return that is allowed to access it.
            return true;
        },

        disabledComponentsCheck: function(component_list) {
            /*
            * Set currently disabled components. Make sure to check if some are active before that so we can disable them.
            * */
            if (!this.toggleInProgress) {
                // Receives list of sidebar components which should be currently disabled.
                // In case one of the disabled components is currently active we need to hide it right away.
                for (var i = 0; i < component_list.length; i++) {
                    var currentComponent = component_list[i];
                    if (this.lastView.type == currentComponent) {
                        // Once we find the component call its toggle method to hide it and break out from the loop.
                        var currentToggleMethod = this.sidebarTypeToggleMethods[currentComponent];
                        this[currentToggleMethod]();
                        break;
                    }
                }

                // If search is disabled on the current course, update title which will be shown in the tooltip
                const isSearchDisabled = component_list.includes(this.sidebarTypes.search);
                const searchTitle = isSearchDisabled ? 'Search is disabled during this activity.' : 'Search';

                this.$el.find('.search-li-sidebar').attr('data-original-title', searchTitle);
            }
            // Update the disabled types list after the check to avoid funky behavior.
            this.disabledTypes = component_list;
        },

        afterToggleCheck: function() {
            // We run this after current toggle is done to not clog the request queue.
            // In case disabled was not called so far cause of progress we make sure its called now.
            // Clear disabled list and run it again to make sure it doesn't block toggle close actions.
            this.toggleInProgress = false;
            if (this.disabledTypes.length) {
                var disabledTypes = this.disabledTypes;
                this.disabledTypes = [];
                this.disabledComponentsCheck(disabledTypes);
            }
        },

        checkExpandedStatus: function(view_type, status) {
            // We don't want to allow big expansion for settings view
            if (this.expandStatus && view_type != this.sidebarTypes.settings && status)
                this.expandViewAnimateShow();
            else
                this.expandViewAnimateHide();
        },

        toggleSyllabus: function() {
            this.disableAllHTML();
            this.pos = window.pageYOffset;
            if (!this.toggleViewPreProcess(this.sidebarTypes.syllabus)) {
                window.scrollTo(0, this.pos);
                return;
            }
            this.appRouter.navigate("syllabus");
            // Create syllabus instance, run animation and listen to events for bookmarks
            this.toggleInProgress = true;
            var _this = this;
            require(["sidebar_syllabus"], function (SyllabusView) {
                _this.syllabusView = new SyllabusView({dataOptions: _this.dataOptions, expandOptions: _this.expandOptions()});
                _this.onShowAnimation(_this.syllabusView, _this.sidebarTypes.syllabus);
                _this.syllabusView.on("concept:Bookmarked", function(data) {
                    _this.saveBookmark(data.concept, data.status, data.element);
                });
                _this.afterToggleCheck();
            });
        },


        toggleSearch: function() {
            if (!searchIsEnabledOnAPlatformLevel) {
                return;
            }
            this.disableAllHTML();
            this.pos = window.pageYOffset;

            const {
                searchCollection,
                searchModel,
                dataOptions
            } = this;

            if (!this.toggleViewPreProcess(this.sidebarTypes.search)) {
                window.scrollTo(0, this.pos);
                return;
            }
            this.appRouter.navigate("search");

            // Create search instance, run animation and listen to events for bookmarks
            this.toggleInProgress = true;
            const _this = this;
            require(["sidebar_search"], function (SearchView) {

                if (searchCollection && searchModel) {
                    _this.searchView = new SearchView({
                        dataOptions,
                        expandOptions: _this.expandOptions(),
                        collection: searchCollection,
                        model: searchModel
                    });
                } else {
                    _this.searchView = new SearchView({
                        dataOptions,
                        expandOptions: _this.expandOptions()
                    });
                    _this.searchCollection = _this.searchView.collection;
                    _this.searchModel = _this.searchView.model;
                }

                _this.onShowAnimation(_this.searchView, _this.sidebarTypes.search);
                _this.afterToggleCheck();
            });
        },

        togglePeerHelp: function() {
            this.disableAllHTML();
            this.pos = window.pageYOffset;
            if (!this.toggleViewPreProcess(this.sidebarTypes.peerHelp)) {
                window.scrollTo(0, this.pos);
                return;
            }
            this.appRouter.navigate("peerHelp");
            this.toggleInProgress = true;
            var _this = this;
            require(["sidebar_peerhelp"], function (PeerHelpView) {
                _this.peerHelpView = new PeerHelpView({dataOptions: _this.dataOptions, expandOptions: _this.expandOptions()});
                _this.onShowAnimation(_this.peerHelpView, _this.sidebarTypes.peerHelp);
                _this.afterToggleCheck();
            });
        },

        toggleGlossary: function() {
            this.disableAllHTML();
            this.pos = window.pageYOffset;
            if (!this.toggleViewPreProcess(this.sidebarTypes.glossary)) {
                window.scrollTo(0, this.pos);
                return;
            }
            this.appRouter.navigate("glossary");
            this.toggleInProgress = true;
            var _this = this;
            require(["sidebar_glossary"], function (GlossaryView) {
                _this.glossaryView = new GlossaryView({dataOptions: _this.dataOptions, expandOptions: _this.expandOptions()});
                _this.onShowAnimation(_this.glossaryView, _this.sidebarTypes.glossary);
                _this.afterToggleCheck();
            });
        },

        toggleTeamDiscussion: function() {
            this.disableAllHTML();
            this.pos = window.pageYOffset;
            if (!this.toggleViewPreProcess(this.sidebarTypes.teamDiscussion)) {
                window.scrollTo(0, this.pos);
                return;
            }
            this.appRouter.navigate("teamDiscussion");
            this.toggleInProgress = true;
            var _this = this;
            require(["sidebar_teamdiscussion"], function (TeamDiscussionView) {
                _this.teamDiscussionView = new TeamDiscussionView({dataOptions: _this.dataOptions, expandOptions: _this.expandOptions()});
                _this.onShowAnimation(_this.teamDiscussionView, _this.sidebarTypes.teamDiscussion);
                _this.afterToggleCheck();
            });
        },

        toggleSettings: function() {
            this.disableAllHTML();
            this.pos = window.pageYOffset;
            if (!this.toggleViewPreProcess(this.sidebarTypes.settings)) {
                window.scrollTo(0, this.pos);
                return;
            }
            this.appRouter.navigate("settings");
            this.toggleInProgress = true;
            var _this = this;
            require(["sidebar_settings"], function (SettingsView) {
                _this.settingsView = new SettingsView();
                _this.onShowAnimation(_this.settingsView, _this.sidebarTypes.settings);
                _this.afterToggleCheck();
            });
        },

        toggleViewPreProcess: function(view_type) {
            var status = true;
            // Run view type animation
            this.handleSidebarItemAnimation(view_type);
            // We don't want to create an instance of this type once again if its created
            // Check is this type of view allowed to be displayed
            if (!this.viewEnabled(view_type) || this.checkLastView(view_type)) {
                status = false;
                this.appRouter.navigate();
            }
            // Check expanded
            this.checkExpandedStatus(view_type, status);
            return status;
        },

        onShowAnimation: function(view, type) {
            this.region.empty({preventDestroy: true});
            this.region.show(view, {preventDestroy: true});
            this.setLastView(view, type);
        },

        onCloseAnimation: function() {
            this.region.empty({preventDestroy: true});
            this.region.onDestroyAnimation();
            this.setLastView(null, null);
        },

        handleSidebarItemAnimation: function(viewType) {
            if (this.disabledTypes.includes(viewType)) return;

            var elementContainer = this.$el.find("#" + viewType).parent();
            var currentElementActive = elementContainer.hasClass(this.menuItemCurrentClass);
            this.$el.find("." + this.menuItemElementClass).removeClass(this.menuItemCurrentClass);
            // Only mark as active if element wasn't active before
            if (!currentElementActive)
                elementContainer.addClass(this.menuItemCurrentClass);
        },

        saveBookmark: function(conceptId, status) {
            var _this = this;
            Backbone.ajax({
                type: "POST",
                dataType: "json",
                url: "/api/v1/courseware/concept/state/bookmark",
                contentType: "application/json; charset=utf-8",
                data: JSON.stringify({"concept": conceptId, "bookmark": status, "course_instance": _this.dataOptions.courseId}),
                    success: function () {
                        // If we were saving state of the currently active concept
                        if (conceptId == _this.dataOptions.conceptId)
                            _this.setBookmark(status);
                        // If syllabus view is currently open
                        if (_this.lastView.type == _this.sidebarTypes.syllabus)
                            _this.lastView.view.setBookmark(conceptId, status);
                    }
            });
        },

        setBookmark: function(status) {
            this.$el.find("#tools_bookmark").toggleClass("bookmark_active", status);
        },

        bookmarkConcept: function(ev) {
            var status = !$(ev.currentTarget).hasClass("bookmark_active");
            this.saveBookmark(this.dataOptions.conceptId, status);
        },

        toggleDraw: function (ev) {
            // We don't allow drawing when in max expand status
            if (this.expandStatus && this.lastView.type != null)  return;
            if (!this.student_tools) {
                this.loadStudentTools("toggleDraw", ev);
                return;
            }
            ev.preventDefault();
            this.toggleViewPreProcess(this.lastView.type);
            this.$el.find("#color_choice").animate({ "left": "-350px" }, "fast");
            this.$el.find("#draw").tooltip("disable");
            this.$el.find("#drawChoice").animate({ "left": "0" }, "fast");

            $("#sticky-container").mouseenter(function() {
                $("#info_draw").fadeOut(1000);
                $("#info_edit_draw").fadeOut(1000);
                $("#info_erase").fadeOut(1000);
            });
        },

        beginDraw: function (ev) {
            ev.preventDefault();
            this.$el.find("#draw>div").removeClass();
            this.$el.find("#draw").tooltip("enable");
            var drawClass = $(ev.currentTarget).children().attr("class");
            this.$el.find("#draw").tooltip("enable");
            this.$el.find("#draw").addClass("active");
            this.$el.find("#stop_editing").show();
            this.showDarkLayer(true);

            var item ='draw-tools_' + drawClass;
            this.metricsChannel.trigger('sidebar_item_click', item);

            if (drawClass != "freehand_white") {
                drawMode = this.drawingModes[drawClass];
                $("#"+drawMode["element"]).show().fadeIn(1000);
                this.$el.find("#draw>div").addClass(drawClass);
                this.$el.find("#drawChoice").animate({ "left": "-250px" }, "fast");
                this.student_tools.freehandMode(drawMode["status"], drawMode["line"]);
            } else {
                this.showDarkLayer(false);
                this.$el.find("#stop_editing").hide();
                this.disableAllHTML();
            }
        },

        colorMenu: function () {
            // We don't allow color choice when in max expand status
            if (this.expandStatus && this.lastView.type != null)  return;
            if (!this.student_tools) {
                this.loadStudentTools("colorMenu", null);
                return;
            }
            this.$el.find("#drawChoice").animate({ "left": "-250px" }, "fast");
            this.toggleViewPreProcess(this.lastView.type);
            var toolsMenu = this.$el.find("#color_choice");
            if (toolsMenu.css("left") == "-350px")
                toolsMenu.animate({ "left": "0" }, "fast");
            else
                toolsMenu.animate({ "left": "-350px" }, "fast");
        },

        colorMenuElements: function (ev) {
            var colorChoice = $(ev.currentTarget).find("input");
            var toolsMenu = this.$el.find("#color_choice");
            var toolsActiveColor = this.$el.find("#tools_color_active");
            toolsActiveColor.css("background-color", colorChoice.data("color"));
            toolsMenu.animate({ "left": "-350px" }, "fast");
            this.student_tools.changeColor(colorChoice);

            var item ='color_' + colorChoice.data("color");
            this.metricsChannel.trigger('sidebar_item_click', item);
        },

        colorPicker: function (ev) {
            if (!this.student_tools) {
                this.loadStudentTools("colorPicker", ev);
                return;
            }
            ev.preventDefault();
            ev.stopPropagation();
            var that = this.$el.find('#color_choice>input:radio + label:not(.disabled)');
            if (that) {
                this.student_tools.changeColor($('#' + that.attr('for')));
                that.click();
            }
        },

        stopEditing: function () {
            this.disableAllHTML();
            this.showDarkLayer(false);
            this.$el.find("#stop_editing").hide();
        },

        addStickyNote: function (ev) {
            // We don't allow sticky notes when in max expand status
            if (this.expandStatus && this.lastView.type != null)  return;
            if (!this.student_tools) {
                this.loadStudentTools("addStickyNote", ev);
                return;
            }
            this.toggleViewPreProcess(this.lastView.type);
            this.disableAllHTML();
            this.showDarkLayer(false);
            this.$el.find("#stop_editing").hide();
            this.student_tools.addNote();
        },

        showDarkLayer: function (status) {
        var layer = $("#darkLayer");
        if (status) {
                layer.show();
            } else {
                layer.hide();
            }
        },

        disableAllHTML: function () {
            this.$el.find("#drawChoice").animate({ "left": "-250px" }, "fast");
            this.$el.find("#draw>div").removeClass();
            this.$el.find("#draw>div").addClass("freehand_white");
            this.$el.find("#draw").removeClass("active");
            this.$el.find("#color_choice").animate({ "left": "-350px" }, "fast");

            $("#info_draw").hide();
            $("#info_erase").hide();
            $('#freehandClearContainer').hide();

            if (this.student_tools)
                this.student_tools.disableAll();
        },

        processConceptUrl: function(url) {
            if (this.lastView.type)
                url = url + "#" + this.lastView.type;
            return url;
        },

        listenTEP: function() {
            var _this = this;
            TEP.addEventListener("TEP.AllTEIReady", function(evt, data) {
                _this.tepReady = true;
                _this.positionToTei();
                _this.studentToolsInitial();
            });
        },

        positionToTei: function() {
            // Scroll to specific teaching element if tei_id is provided as URL parameter and teaching elements are loaded.
            if (this.currentTeachingElement && this.tepReady)
                TEP.positionToTei(this.currentTeachingElement);
        },

        studentToolsInitial: function() {
            // Check if there is any initial data and if so load student tools
            if (this.dataOptions.drawings != this.drawingsEmpty || this.dataOptions.notes != false)
                this.loadStudentTools(null, null);
        },

        loadStudentTools: function(callback_method, callback_parameter) {
            // TODO - in concept redesign we plan to remove student tools - remove this method and its uses at that time
            if (window.useStudentTools && !this.student_tools) {
                var _this = this;
                require(["studenttools"], function (StudentTools) {
                    _this.student_tools = new StudentTools();
                    if (callback_method)
                        _this[callback_method](callback_parameter);
                });
            }
        },

        modulesContainer: function() {
            return this.$el.find(this.regions.utilityPane);
        },

        expandOptions: function() {
            return {
                status: this.expandStatus,
                animationSpeed: this.expandSpeed,
                widthMin: this.expandWidthMin,
                widthMax: this.expandWidthMax
            };
        },

        expandView: function(ev) {
            var currentExpandElement = $(ev.currentTarget).children();
            this.expandStatus = !this.expandStatus;
            if (this.expandStatus) {
                this.expandViewAnimateShow();
                currentExpandElement.removeClass("fa-expand").addClass("fa-compress");
                this.lastView.view.expandShow();
            } else {
                this.expandViewAnimateHide();
                currentExpandElement.removeClass("fa-compress").addClass("fa-expand");
                this.lastView.view.expandHide();
            }
        },

        dispatchDiscussionTeiEvent: function(discussion_id) {
            // Dispatch event with discussion id so that in case discussion tei is on the page it can refresh its data as well
            var discussionChangeEvent = new CustomEvent("discussion-" + discussion_id + "-updated");
            document.body.dispatchEvent(discussionChangeEvent);
        },

        expandViewAnimateShow: function() {
            this.modulesContainer().animate({"width": this.expandWidthMax+"px"}, this.expandSpeed);

            // Block right side of the page (teaching elements interaction) while expanded.
            this.teachingElementContainer.css("pointer-events", "none").animate({"opacity": this.expandOpacityMin}, this.expandSpeed);
            this.titlesContainer.css("pointer-events", "none").animate({"opacity": this.expandOpacityMin}, this.expandSpeed);

            this.body.css("overflow", "hidden");
            if (this.expandAnimateOpElements)
                this.expandAnimateOpElements.css("pointer-events", "none").animate({"opacity": this.expandOpacityMin}, this.expandSpeed);
        },

        expandViewAnimateHide: function() {
            this.modulesContainer().animate({"width": this.expandWidthMin+"px"}, this.expandSpeed);

            // Unblock right side of the page (teaching elements interaction).
            this.teachingElementContainer.css("pointer-events", "").animate({"opacity": this.expandOpacityMax}, this.expandSpeed);
            this.titlesContainer.css("pointer-events", "").animate({"opacity": this.expandOpacityMax}, this.expandSpeed);

            this.body.css("overflow", "");
            if (this.expandAnimateOpElements)
                this.expandAnimateOpElements.css("pointer-events", "").animate({"opacity": this.expandOpacityMax}, this.expandSpeed);
        },

        sendMetric: function(ev) {
            var item = $(ev.currentTarget).data('metric-item');
            this.metricsChannel.trigger('sidebar_item_click', item);
        }
    });

    return SidebarView;
});
