define([
    'jquery',
    'underscore',
    'backbone'
], function(
    $,
    _,
    Backbone
) {

    return Backbone.Model.extend({
        initialize: function () {
            this.on('change', this.updateModelAttributes)
        },
        defaults: {
            groups: [],
            photo: "/bcore/static/common/img/sample/icon-person-x6.png"
        },

        urlRoot: function() {
            return "/student/" + (this.get("customUrl") || "api");
        },

        getAllOrganizationsRows: function () {
            const organizations = this.get('organization');
            if (!organizations) return;

            return organizations.map(organization => ({
                    name: organization.name,
                    positionLevel: organization.position_level,
                    positionTitle: organization.position_title,
                    dates: this.getDatesString(organization),
                }));
        },

        getAllEducationsRows: function () {
            const educations = this.get('education');
            if (!educations) return;

            return educations.map(education => ({
                    school: education.school,
                    degree: education.degree,
                    fieldOfStudy: education.field_of_study,
                    dates: this.getDatesString(education),
                }));
        },

        updateModelAttributes: function() {
            this.set({ organizationRows: this.getAllOrganizationsRows() });
            this.set({ educationRows: this.getAllEducationsRows() });
        },

        /**
         * Covers three cases:
         *  (1) If there are both year_from and year_to, expose them.
         *  (2) If there is only year_from, it means year_to was chosen to be 'Present'
         *  (3) No years are specified
         * @param entity
         */
        getDatesString: function(entity) {
            const from = entity.year_from;
            const to = entity.year_to;

            if (from && to)
                return `• ${from} - ${to}`;
            else if (from)
                return `• ${from} - Present`;
        },

        /**
         * Invalid cases:
         * (1) End date exists but not the start date
         * (2) Start date exists but not the end date (includes end date != 'present')
         * (3) Both dates exist but start date is greater than the end date.
         * @param yearFrom
         * @param yearTo
         * @returns {string} - contains invalid message. If validation passes, returns 'undefined'.
         */
        validateDateFields: function(yearFrom, yearTo) {
            if (isNaN(yearFrom) && (!isNaN(yearTo) || yearTo === 'present'))
                return 'End date cannot be selected if the start date is not selected.';
            else if (!isNaN(yearFrom) && isNaN(yearTo) && yearTo !== 'present')
                return 'End date must be selected if the start date is selected.';
            else if (yearFrom > yearTo)
                // At this point we are sure both dates are either number or left empty (meaning strings
                // 'from' and 'to')
                return 'End date must be greater than start date.';
        }
    });
});
