/**
 * This file contains utilities to help in the webpack transition.
 */
define(function() {
  return {
    isExecutingInBundle: typeof __webpack_require__ === 'function',

    /**
     * This function adapts the RequireJS.require( file, callback) style to promises.
     * This is used to make functions that behave the same as
     * webpack / ES6's dynamic import(), which also returns a promise:
     * https://webpack.js.org/guides/code-splitting/#dynamic-imports
     *
     * @param fileToRequire - any path you would normally pass to RequireJS
     * @returns {Promise<Module>}
     * Note - this promise resolves to a module. You can catch the error to handle module timeout errors.
     */
    getRequireJsPromise: (fileToRequire) => new Promise((resolve, reject) => {
      try {
        require([fileToRequire], (requiredFile) => resolve(requiredFile));
      } catch (err) {
        reject(`${fileToRequire} failed to load in WebpackUtils.getRequireJsPromise`, err);
      }
    }),
  };
});
