var BcoreGlobal = {
    serializedDateFormat: 'yyyy-MM-ddTHH:mm:ssZ',
    momentDateFormat: 'MM/DD/YYYY hh:mm A',
    momentDefaultZone: 'America/New_York',
    momentDefaultZoneOffset: 240,
    displayDateFormat: 'MM/dd/yyyy h:mm tt',
    displayDateFormatOnlyDate: 'MM/dd/yyyy',
    bootstrapDatePickerFormat: 'MM/dd/yyyy hh:mm',
    maximumUploadSize: 10485760,
    htmlEncodeMap: {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;'
    },

    getDifficultyDisplay: function(value) {
        var difficulty = Number(value);
        switch(difficulty) {
            case 0:
                return 'Low';
            case 2:
                return 'High';
            default:
                return 'Medium';
        }
    },
    formatStudentDate: function(dateString) {
        var dateObj = new Date(dateString);
        return dateObj.toDateString() +
            ', ' + dateObj.getHours() + ':' + (dateObj.getMinutes() < 10 ? '0' : '') + dateObj.getMinutes();
    },
    htmlEncode: function(value) {
        var result = String(value);
        for (var c in BcoreGlobal.htmlEncodeMap) {
            var re = new RegExp(c, "g");
            result = result.replace(re, BcoreGlobal.htmlEncodeMap[c]);
        }
        return result;
    },

    htmlDecode: function(value) {
        var result = String(value);
        for (var c in BcoreGlobal.htmlEncodeMap) {
            var re = new RegExp(BcoreGlobal.htmlEncodeMap[c], "g");
            result = result.replace(re, c);
        }
        return result;
    },

    htmlEncodeObject: function(obj) {
        if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType('')) {
            obj = BcoreGlobal.htmlEncode(obj);
        } else if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType([])) {
            for (var i = 0; i < obj.length; i++) {
                obj[i] = BcoreGlobal.htmlEncodeObject(obj[i]);
            }
        } else if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType({})) {
            for (var property in obj) {
                obj[property] = BcoreGlobal.htmlEncodeObject(obj[property]);
            }
        }
        return obj;
    },

    htmlDecodeObject: function(obj) {
        if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType('')) {
            obj = BcoreGlobal.htmlDecode(obj);
        } else if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType([])) {
            for (var i = 0; i < obj.length; i++) {
                obj[i] = BcoreGlobal.htmlDecodeObject(obj[i]);
            }
        } else if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType({})) {
            for (var property in obj) {
                obj[property] = BcoreGlobal.htmlDecodeObject(obj[property]);
            }
        }
        return obj;
    },

    getJavascriptType: function(o) {
        return Object.prototype.toString.call(o);
    },

    findValueByKey: function(obj, key) {
        var result;
        if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType({})) {
            if (key in obj) {
                result = obj[key];
            } else {
                for (var k in obj) {
                    result = BcoreGlobal.findValueByKey(obj[k]);
                    if (result) {
                        break;
                    }
                }
            }
        } else if (Object.prototype.toString.call(obj) === BcoreGlobal.getJavascriptType([])) {
            for (var i = 0; i < obj.length; i++) {
                result = BcoreGlobal.findValueByKey(obj[i]);
                if (result) {
                    break;
                }
            }
        }
        return result;
    },

    findQuestionText: function(input) {
        var result = BcoreGlobal.findValueByKey(input, 'question');
        if (result) {
            if (Object.prototype.toString.call(result) === BcoreGlobal.getJavascriptType({})) {
                return BcoreGlobal.findValueByKey(result, 'content') || '';
            } else {
                return result;
            }
        }

        return BcoreGlobal.findValueByKey(input, 'title') || '';
    }

};

function isFunction(functionToCheck) {
 var getType = {};
 return functionToCheck && getType.toString.call(functionToCheck) === '[object Function]';
}

if (typeof define === "function") {
  define("BcoreGlobal", function(require) {
    return BcoreGlobal;
  });
}
