define([
    'jquery'
], function ( $ ) {

    window.Bcore = {
        serializedDateFormat: 'yyyy-MM-ddTHH:mm:ssZ',
        momentDateFormat: 'MM/DD/YYYY hh:mm A',
        momentDefaultZone: 'America/New_York',
        momentDefaultZoneOffset: 240,
        displayDateFormat: 'MM/dd/yyyy h:mm tt',
        displayDateFormatOnlyDate: 'MM/DD/YYYY',
        bootstrapDatePickerFormat: 'MM/dd/yyyy hh:mm',
        maximumUploadSize: 10485760,
        htmlEncodeMap: {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;'
        },
        scriptEncodeMap: {
            '<script>': '&lt;script&gt;',
            '</script>': '&lt;/script&gt;',
            '<style>': '&lt;style&gt;',
            '</style>': '&lt;/style&gt;'
        },

        getDifficultyDisplay: function (value) {
            var difficulty = Number(value);
            switch (difficulty) {
                case 0:
                    return 'Low';
                case 2:
                    return 'High';
                default:
                    return 'Medium';
            }
        },

        htmlEncode: function (value) {
            var result = String(value);
            for (var c in Bcore.htmlEncodeMap) {
                var re = new RegExp(c, "g");
                result = result.replace(re, Bcore.htmlEncodeMap[c]);
            }
            return result;
        },

        htmlDecode: function (value) {
            var result = String(value);
            for (var c in Bcore.htmlEncodeMap) {
                var re = new RegExp(Bcore.htmlEncodeMap[c], "g");
                result = result.replace(re, c);
            }
            return result;
        },

        htmlDecodeUnsafe: function (value) {
            var result = String(value);
            for (var c in Bcore.htmlEncodeMap) {
                var re = new RegExp(Bcore.htmlEncodeMap[c], "g");
                result = result.replace(re, c);
            }
            for (var c in Bcore.scriptEncodeMap) {
                var re = new RegExp(c, "g");
                result = result.replace(re, Bcore.scriptEncodeMap[c]);
            }
            return result;
        },

        htmlEncodeObject: function (obj) {
            if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType('')) {
                obj = Bcore.htmlEncode(obj);
            } else if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType([])) {
                for (var i = 0; i < obj.length; i++) {
                    obj[i] = Bcore.htmlEncodeObject(obj[i]);
                }
            } else if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType({})) {
                for (var property in obj) {
                    obj[property] = Bcore.htmlEncodeObject(obj[property]);
                }
            }
            return obj;
        },

        htmlDecodeObject: function (obj) {
            if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType('')) {
                obj = Bcore.htmlDecode(obj);
            } else if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType([])) {
                for (var i = 0; i < obj.length; i++) {
                    obj[i] = Bcore.htmlDecodeObject(obj[i]);
                }
            } else if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType({})) {
                for (var property in obj) {
                    obj[property] = Bcore.htmlDecodeObject(obj[property]);
                }
            }
            return obj;
        },

        scriptEncode: function (value) {
            var result = String(value);
            for (var c in Bcore.scriptEncodeMap) {
                var re = new RegExp(c, "g");
                result = result.replace(re, Bcore.scriptEncodeMap[c]);
            }
            return result;
        },

        getJavascriptType: function (o) {
            return Object.prototype.toString.call(o);
        },

        findValueByKey: function (obj, key) {
            var result;
            if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType({})) {
                if (key in obj) {
                    result = obj[key];
                } else {
                    for (var k in obj) {
                        result = Bcore.findValueByKey(obj[k]);
                        if (result) {
                            break;
                        }
                    }
                }
            } else if (Object.prototype.toString.call(obj) === Bcore.getJavascriptType([])) {
                for (var i = 0; i < obj.length; i++) {
                    result = Bcore.findValueByKey(obj[i]);
                    if (result) {
                        break;
                    }
                }
            }
            return result;
        },

        findQuestionText: function (input) {
            var result = Bcore.findValueByKey(input, 'question');
            if (result) {
                if (Object.prototype.toString.call(result) === Bcore.getJavascriptType({})) {
                    return Bcore.findValueByKey(result, 'content') || '';
                } else {
                    return result;
                }
            }

            return Bcore.findValueByKey(input, 'title') || '';
        }
    };

    /*
     Append trailing slash to all AJAX requests to fix interaction
     with Tastypie API
     */
    $(document).ajaxSend(function (event, xhr, options) {
        var parts = options.url.split('?');
        var url = parts[0];
        if (url.lastIndexOf('/') != (url.length - 1)) {
            url = url + '/';
        }
        options.url = url + (parts.length > 1 ? '?' + parts[1] : '');
    });

    /*
     Global check for unauthorized AJAX requests (401)
     Forcing page refresh to get login credentials
     */
    $(document).ajaxError(function (e, xhr, options) {
        if(xhr.status == 401) {
            if (xhr.responseText == 'Session Timeout') {
                window.location = '/session_timeout';
            } else {
                window.location.reload();
            }
        }
    });

});
