'use strict';
var require = {

  // By default, all modules are located relative to this path. If baseUrl
  // is not explicitly set, then all modules are loaded relative to
  // the directory that holds the build file. If appDir is set, then
  // baseUrl should be specified as relative to the appDir.
  baseUrl: '/bcore/static',

  // In case app version is available attach it to all files being loaded by require.
  // This is to ensure student gets the latest script versions in case of production updates.
  urlArgs: function(id, url) {
    var args = "";
    if (window.version_tag_pure)
      args = (url.indexOf('?') === -1 ? '?' : '&') + "v=" + window.version_tag_pure;
    return args;
  },

  // Set paths for modules. If relative paths, set relative to baseUrl above.
  // If a special value of 'empty:' is used for the path value, then that
  // acts like mapping the path to an empty file. It allows the optimizer to
  // resolve the dependency to path, but then does not include it in the output.
  // Useful to map module names that are to resources on a CDN or other
  // http: URL when running in the browser and during an optimization that
  // file should be skipped because it has no dependencies.
  paths: {
    tep: 'tep-jsl/0.1',
    ConceptCodesplits: "common/js/ConceptCodesplitsRequireJs",
    sidebar: 'courseware/js/app/sidebar',
    courseware_glossary: 'courseware/glossary',
    jquery: 'common/js/vendor/jquery/jquery',
    jqueryui: 'common/js/vendor/jquery/jquery-ui',
    jqueryui_with_tooltip: 'common/js/vendor/jquery/jquery-ui-with-tooltip',
    jquery_ui_touch_punch: 'common/js/vendor/jquery_plugins/jquery_ui_touch_punch',
    spread: 'common/js/vendor/spread',
    underscore: 'common/js/vendor/marionette/underscore.min',
    backbone: 'common/js/vendor/marionette/backbone.min',
    // backbone: 'https://cdnjs.cloudflare.com/ajax/libs/backbone.js/1.2.3/backbone',
    marionette: 'common/js/vendor/marionette/backbone.marionette.min',
    'backbone.radio': 'common/js/vendor/marionette/backbone.radio.min',
    // marionette: 3.0.0
    'marionette3': 'common/js/vendor/marionette3/backbone.marionette',
    'marionette3_min': 'common/js/vendor/marionette3/backbone.marionette.min',
    // marionette: 'https://cdnjs.cloudflare.com/ajax/libs/backbone.marionette/2.4.3/backbone.marionette',
    peerassessment: 'peerassessment/js',
    parsley: 'common/js/vendor/parsley',
    // parsley: 'https://cdnjs.cloudflare.com/ajax/libs/parsley.js/2.4.3/parsley.min',
    'parsley.min': 'common/js/vendor/parsley.min',
    date: 'common/js/vendor/date',
    tags_input: 'common/js/vendor/tags_input/jquery.tagsinput',
    bootstrap_typeahead: 'common/vendor/bootstrap-typeahead/js/bootstrap3-typeahead.min',
    magicsuggest: 'common/js/vendor/magicsuggest/magicsuggest-min',
    wordcloud: 'common/js/vendor/wordcloud2',
    link: 'common/js/app/link',
    jquery_cookie: 'common/js/vendor/jquery_plugins/jquery.cookie',
    jquery_knob: 'common/js/vendor/jquery_plugins/jquery.knob',
    csrf: 'common/js/app/csrf',
    text: 'common/js/vendor/marionette/text',
    hbx_snapshot: 'dist/js/common/hbxsnapshot.bundle.min',
    hbxclient: 'dist/js/common/hbxclient.bundle.min',
    // Bootstrap version 3.3.6
    bootstrap: 'common/vendor/bootstrap/js/bootstrap.min',
    // Bootstrap date time picker
    bootstrap_datetime_picker: 'common/vendor/bootstrap-datetimepicker/js/bootstrap-datetimepicker.min',
    // custom Datatable version 1.10.11
    'datatables.net': 'common/vendor/DataTables-1.10.11/jquery.dataTables.min',
    'datatables.net-bs': 'common/vendor/DataTables-1.10.11/dataTables.bootstrap.min',
    datatables: 'common/vendor/DataTables-1.10.11/datatables',
    domReady: 'common/js/vendor/domReady',
    html2canvas: 'common/js/app/html2canvas',
    TeachingElementInstance: 'creator/js/models-require/TeachingElementInstance',
    TeachingElementInstanceGroup: 'creator/js/models-require/TeachingElementInstanceGroup',
    TeachingElement: 'creator/js/models-require/TeachingElement',
    CourseVersion: 'creator/js/models-require/CourseVersion',
    Course: 'creator/js/models-require/Course',
    CoursewareContainer: 'creator/js/models-require/CoursewareContainer',
    CourseVersionPathContainer: 'creator/js/models-require/CourseVersionPathContainer',
    ModalView: 'common/js/views-require/ModalView',
    // Moment.js version : 0.5.0
    moment: 'common/vendor/momentjs/moment-with-locales',
    moment_timezone: 'common/vendor/momentjs/moment-timezone-with-data',
    mathjax: 'common/js/vendor/mathjax/MathJax.js?config=TeX-AMS-MML_HTMLorMML&amp;delayStartupUntil=configured',
    panzoom: 'common/js/vendor/panzoom',
    textarearesize: 'common/js/vendor/jquery_plugins/jquery.autosize',
    tinyMCE: 'common/js/vendor/tinymce/js/tinymce/tinymce.min',
    QuestionListView: 'questionbank/js/views-require/QuestionListView',
    AlertView: 'common/js/views-require/AlertView',
    TeachingElementInstanceFormView: 'creator/js/views-require/TeachingElementInstanceFormView',
    FileUploader: 'common/js/views-require/FileUploader',
    FileUploadView: 'common/js/views-require/FileUploadView',
    Question: 'questionbank/js/models-require/Question',
    QuestionFormView: 'questionbank/js/views-require/QuestionFormView',
    localscroll: 'common/js/vendor/jquery_plugins/jquery.localscroll',
    scrollTo: 'common/js/vendor/jquery_plugins/jquery.scrollTo',
    cropit: 'common/js/vendor/jquery_plugins/jquery.cropit.min',
    markjs: 'common/js/vendor/jquery_plugins/jquery.mark.min',
    hColumns: 'common/vendor/hcolumns/js/jquery.hcolumns',
    uuid: 'common/js/vendor/uuid',
    creatorUtils: 'creator/js/creator-utils',
    creatorTableUtils: 'creator/js/creator-table-utils',
    mapbox: 'common/js/vendor/mapbox.standalone',
    fabric: 'common/js/vendor/fabric',
    stickynotes: 'common/js/vendor/jquery_plugins/jquery.stickynotes',
    modernizr_pointerevents: 'common/js/vendor/modernizr/modernizr.pointerevents',
    jquery_resize: 'common/js/vendor/jquery_plugins/jquery.ba-resize',
    Keyword: 'courseware/glossary/models/Keyword',
    StudentLog: 'courseware/js/app/models/StudentLog',
    KeywordList: 'courseware/glossary/models/KeywordList',
    Glossary: 'courseware/glossary/models/Glossary',
    GlossaryStudentModalView: 'courseware/glossary/views/GlossaryStudentModalView',
    dropdown: 'common/js/vendor/dropdown',
    Conversation: 'messages/js/models/Conversation',
    SharedArticle: 'social/js/models/SharedArticle',
    StudentLanguage: 'student/js/models/StudentLanguage',
    StudentLanguages: 'student/js/collections/StudentLanguages',
    Education: 'student/js/models/Education',
    Educations: 'student/js/collections/Educations',
    filterDate: 'audit/js/filterDate',
    compareDates: 'audit/js/compareDates',
    // Bootstrap Datetime picker version : 4.15.35
    dateTimePicker: 'common/vendor/bootstrap-datetimepicker/js/bootstrap-datetimepicker.min',
    StudentOrganization: 'student/js/models/StudentOrganization',
    StudentOrganizations: 'student/js/collections/StudentOrganizations',
    tastyBackbonePie: 'common/js/vendor/tastybackbonepie-min',
    ThingAbout: 'student/js/models/ThingAbout',
    ThingsAbout: 'student/js/collections/ThingsAbout',
    socketIO: 'common/js/vendor/socket.io',
    dashboard_base: 'courseware/js/dashboard-base',
    DstratTour: 'tours/js/models/DstratTour',
    CoreMapTour: 'tours/js/models/CoreMapTour',
    CoreDashboardTour: 'tours/js/models/CoreDashboardTour',
    CoreSyllabusTour: 'tours/js/models/CoreSyllabusTour',
    CoreConceptTour: 'tours/js/models/CoreConceptTour',
    ActivityFeedListView: 'courseware/activity/views/ActivityFeedListView',
    ActivityFeedItemView: 'courseware/activity/views/ActivityFeedItemView',
    AnnouncementActivityExpandedView: 'courseware/activity/views/AnnouncementActivityExpandedView',
    PeerHelpActivityExpandedView: 'courseware/activity/views/PeerHelpActivityExpandedView',
    StarsActivityExpandedView: 'courseware/activity/views/StarsActivityExpandedView',
    CommentsActivityExpandedView: 'courseware/activity/views/CommentsActivityExpandedView',
    TeamAssignmentsExpandedView: 'courseware/activity/views/TeamAssignmentsExpandedView',
    Activity: 'courseware/activity/models/Activity',
    Activities: 'courseware/activity/models/Activities',
    sly: 'common/js/vendor/sly/sly.min',
    kaltura_low_quality: 'common/js/vendor/kaltura/kaltura_low_quality_31854781',
    // if you need an unminified version of kaltura_low_quality above, comment out the above and use below instead:
    // kaltura_low_quality: 'common/js/vendor/kaltura/kaltura_low_quality_31854781_unminified',
    kaltura_rtmp: 'common/js/vendor/kaltura/kaltura_rtmp_40278101',
    // if you need an unminified version of kaltura_rtmp above, comment out the above and use below instead:
    // kaltura_rtmp: 'common/js/vendor/kaltura/kaltura_rtmp_40278101_unminified',
    kaltura_dstrat: 'common/js/vendor/kaltura/kaltura_dstrat_32104651',
    highcharts: 'common/js/vendor/highcharts/highcharts',
    highcharts_more: 'common/js/vendor/highcharts/highcharts-more',
    highcharts_accessibility: 'common/js/vendor/highcharts/modules/accessibility',
    highcharts_exporting: 'common/js/vendor/highcharts/modules/exporting',
    highcharts_exportData: 'common/js/vendor/highcharts/modules/export-data',

    // ---------------------------------------------------------------------------------------------------- //
    // HBX MODULES SECTION
    // Files defined in this section will be bundled together with their dependencies in a module.
    // This is to save time on loading and executing.
    // To add a bundled file:
    // - define it here to be consistent (nothing major will happen if you don't but to keep it in one place)
    // - add it to modules section in project_root/app.build.js together with files you wish to exclude from build
    //   (usually those you know wil be loaded before the bundle anyway, like jquery, backbone etc)
    // - add path to module in STATIC_REQUIRE_MODULES list in settings. This is VERY IMPORTANT!
    //   This is used to delete the generated modules before attempting to create them again, optimize WILL FAIL IF NOT!

    // Main pages modules.
    map: 'courseware/js/init-map',
    core_dashboard: 'courseware/js/init-core-dashboard',
    dstrat_dashboard: 'courseware/js/init-dstrat-dashboard',
    user_search: 'courseware/js/init-user-search',
    syllabus: 'courseware/js/init-syllabus',
    concept: 'courseware/js/init-concept',
    // Concept sidebar modules
    sidebar_syllabus: 'courseware/js/app/sidebar/syllabus/views/SyllabusView',
    sidebar_search: 'courseware/js/app/sidebar/search/views/SearchView',
    sidebar_peerhelp: 'courseware/js/app/sidebar/peer-help/views/PeerHelpQuestionView',
    sidebar_glossary: 'courseware/js/app/sidebar/glossary/views/GlossaryView',
    sidebar_teamdiscussion: 'courseware/js/app/sidebar/team-discussion/views/DiscussionsView',
    sidebar_settings: 'courseware/js/app/sidebar/settings/views/SettingsView',
    // Map modules. Map_Welcome is the module for left sidebar, and Map_Map is the module for the map itself
    map_welcome: 'courseware/js/app/views/WelcomeView',
    map_map: 'courseware/map/views/MapView',
    // Core Dashboard modules
    wave_core_my_progress: 'courseware/js/app/views/WaveCoreMyProgress',
    // Misc
    RequiredFieldsAlertView: 'courseware/js/app/views/RequiredFieldsAlertView',
    DjangoUnderscoreTemplate: 'common/js/app/DjangoUnderscoreTemplate',
    studenttools: 'common/js/app/hbx_student_tools',
    openGlossary: 'courseware/glossary/open-glossary',
    TourFactory: 'tours/js/TourFactory',
    // Dashboard Team chat module
    dashboard_team_chat: 'courseware/js/app/dashboard_team_chat/views/TeamChatLayoutView',

    // ---------------------------------------------------------------------------------------------------- //

    // Core Dashboard modules
    wave_core_assignments_tab: 'courseware/js/app/core-assignments/AssignmentsTabView',
    bootstrap_transfer: 'common/vendor/bootstrap-transfer/js/bootstrap-transfer',
    bootstrap_submenu: 'common/vendor/bootstrap-submenu/bootstrap-submenu.min',
    // loglevel: 'https://cdnjs.cloudflare.com/ajax/libs/loglevel/1.4.1/loglevel.min.js'
    loglevel: 'common/js/vendor/loglevel.min'
  },
  // If shim config is used in the app during runtime, duplicate the config
  // here. Necessary if shim config is used, so that the shim's dependencies
  // are included in the build. Using 'mainConfigFile' is a better way to
  // pass this information though, so that it is only listed in one place.
  // However, if mainConfigFile is not an option, the shim config can be
  // inlined in the build config.
  shim: {
    parsley: {
      deps: ['jquery']
    },
    bootstrap: {
      deps: ['jquery', 'jqueryui', 'jqueryui_with_tooltip']
    },
    datatables: {
      deps: ['jquery']
    },
    'common/js/vendor/d3_v3.2.8': {
      exports: 'd3'
    },
    jquery_ui_touch_punch: {
      deps: ['jqueryui']
    },
    jquery_resize: {
      deps: ['jquery']
    },
    csrf: {
      deps: ['jquery', 'jquery_cookie']
    },
    'date': {
      deps: []
    },
    'dateTimePicker': {
      deps: ['jquery']
    },
    tinyMCE: {
      exports: 'tinyMCE',
      init: function() {
        this.tinyMCE.DOM.events.domLoaded = true;
        return this.tinyMCE;
      }
    },
    scrollTo: {
      deps: ['jquery']
    },
    cropit: {
      deps: ['jquery']
    },
    localscroll: {
      deps: ['scrollTo']
    },
    hColumns: {
      deps: ['jquery']
    },
    sly: {
      deps: ['jquery']
    },
    mathjax: {
      exports: 'MathJax',
      init: function() {
        MathJax.Hub.Config({
          tex2jax: {
            inlineMath: [['$!', '$!'], ['\\(', '\\)']],
            processEscapes: true,
            ignoreClass: 'teaching-element',
            processClass: 'tex'
          },
          styles: {
            '#MathJax_MSIE_Frame': {
              'position': 'static',
              'height': '0',
              'overflow': 'hidden',
              'visibility': 'hidden',
              'display': 'none'
            }
          },
          showMathMenu: false,
          showMathMenuMSIE: false,
          messageStyle: 'none',
          'HTML-CSS': {
            imageFont: null
          },
          TeX: {
            extensions: ['color.js']
          }
        });
        return MathJax;
      }
    },
    dataTables_bootstrap: {
      deps: ['jquery', 'jqueryui', 'bootstrap', 'datatables']
    },
    tastyBackbonePie: {
      deps: ['jquery', 'backbone', 'underscore']
    },
    jquery_cookie: {
      deps: ['jquery']
    },
    jquery_knob: {
      deps: ['jquery']
    },
    stickynotes: {
      deps: ['jquery', 'jqueryui', 'textarearesize']
    },
    magicsuggest: {
      deps: ['jquery', 'bootstrap']
    },
    highcharts_more: {
      deps: ['highcharts'],
    },
    highcharts_accessibility: {
      deps: ['highcharts'],
    },
    highcharts_exporting: {
      deps: ['highcharts']
    },
    highcharts_exportData: {
      deps: ['highcharts_exporting', 'highcharts']
    },
    Conversation: {
      deps: ['backbone'],
      exports: 'Conversation'
    },
    Education: {
      deps: ['backbone'],
      exports: 'Education'
    },
    Educations: {
      deps: ['backbone'],
      exports: 'Educations'
    },
    StudentLanguage: {
      deps: ['backbone'],
      exports: 'StudentLanguage'
    },
    StudentLanguages: {
      deps: ['backbone'],
      exports: 'StudentLanguages'
    },
    StudentOrganization: {
      deps: ['backbone'],
      exports: 'StudentOrganization'
    },
    StudentOrganizations: {
      deps: ['backbone'],
      exports: 'StudentOrganizations'
    },
    ThingAbout: {
      deps: ['backbone'],
      exports: 'ThingAbout'
    },
    ThingsAbout: {
      deps: ['backbone'],
      exports: 'ThingsAbout'
    },
    StudentLog: {
      deps: ['backbone'],
      exports: 'StudentLog'
    }
  },
  config: {
    text: {
      useXhr: function() {
        return true;
      }
    }
  }
};


// Export requireJS config when loaded as CJS module.
// Used in webpack.config.
if (typeof module === 'object' && module.exports) {
  module.exports = require;
}
