// Generated by CoffeeScript 1.6.3
/*
Event Dispatcher
================

EventDispatcher is a base class that is used by many classes in the TEP-JSL.
It supports basic, object-bound event listening, as opposed to a more generic
pub/sub model.
*/


(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  define(function() {
    var EventDispatcher;
    return EventDispatcher = (function() {
      function EventDispatcher() {
        this.off = __bind(this.off, this);
        this.on = __bind(this.on, this);
        this.dispatchEvent = __bind(this.dispatchEvent, this);
        this.removeEventListener = __bind(this.removeEventListener, this);
        this.addEventListener = __bind(this.addEventListener, this);
      }

      /*
      Event that is fired by TeachingElement.constructor when it discovers
      it contains a term that has been passed to the ?searchPhrase url param.
      Listened to by Drop Bars and Trigger Points to know when to open up.
      */


      EventDispatcher.prototype.INITIAL_SEARCH_RESULT_FOUND = 'TeachingElement.initialSearchResultsFound';

      /*
      API
      ===
      
      addEventListener(evt, callback)
      -------------------------------
      Binds a callback method to the provided event name (string). A wildcard ("*")
      can be used as the event name to bind to all events, though generally this is
      not recommended.
      
      Structures are created lazily to remove reliance on constructor methods.
      */


      EventDispatcher.prototype.addEventListener = function(evt, callback) {
        var _base;
        if (this.__listeners == null) {
          this.__listeners = {};
        }
        if ((_base = this.__listeners)[evt] == null) {
          _base[evt] = [];
        }
        if ((callback != null) && __indexOf.call(this.__listeners[evt], callback) < 0) {
          return this.__listeners[evt].push(callback);
        }
      };

      /*
      
      removeEventListener(evt, callback)
      ----------------------------------
      Unbinds the callback from the provided event name. Wildcards can only unbind
      assignments made by wildcards.
      */


      EventDispatcher.prototype.removeEventListener = function(evt, callback) {
        if ((this.__listeners != null) && (this.__listeners[evt] != null)) {
          return this.__listeners[evt].splice(this.__listeners[evt].indexOf(callback), 1);
        }
      };

      /*
      
      dispatchEvent(evt, data)
      ------------------------
      Fires an event with the provided data payload.
      
      As the event is fired, the data payload is appended with currentTarget and,
      if not yet set, target attributes. currentTarget holds a reference to the
      currently-firing object, whereas target holds a reference to the original
      source object of the event. This distinction is important when events "bubble",
      i.e. a container object listens for events of its children and passes them
      up the chain.
      */


      EventDispatcher.prototype.dispatchEvent = function(evt, data) {
        var cb, list, _i, _j, _len, _len1, _results;
        data.currentTarget = this;
        if (data.target == null) {
          data.target = this;
        }
        if ((this.__listeners != null) && this.__listeners[evt]) {
          list = this.__listeners[evt].slice(0);
          for (_i = 0, _len = list.length; _i < _len; _i++) {
            cb = list[_i];
            cb(evt, data);
          }
        }
        if ((this.__listeners != null) && this.__listeners['*']) {
          list = this.__listeners['*'].slice(0);
          _results = [];
          for (_j = 0, _len1 = list.length; _j < _len1; _j++) {
            cb = list[_j];
            _results.push(cb(evt, data));
          }
          return _results;
        }
      };

      /*
      
      on(evt, callback)
      -----------------
      Alias for addEventListener
      */


      EventDispatcher.prototype.on = function() {
        return this.addEventListener.apply(this, arguments);
      };

      /*
      
      off(evt, callback)
      -----------------
      Alias for removeEventListener
      */


      EventDispatcher.prototype.off = function() {
        return this.removeEventListener.apply(this, arguments);
      };

      return EventDispatcher;

    })();
    /*
    
    Notes
    =====
    
    While similar, the method signatures are not identical to JavaScript's native
    event system, and it might be worthwhile to either make them closer, or
    farther apart.
    */

  });

}).call(this);
