// Generated by CoffeeScript 1.6.3
(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  define(function(require) {
    var $, EventDispatcher, Graph, Model, View, defaults;
    $ = require('jquery');
    EventDispatcher = require('tep/utils/event/event_dispatcher');
    View = require('./view');
    Model = require('./models/model');
    /*
    Draw Graph
    ==========
    
    The Draw Graph provides a Quandrant I static-scale graph.  It supports multiple different
    drawn objects (see ./objects) and drawing and manipulation tools (see ./tools)
    
    Configuration
    =============
    Draw Graph supports the following configurations, passed in as a single config object.
    */

    defaults = {
      /*
      *   title: An optional title displayed above the graph.
      
      Defaults to null.
      */

      title: null,
      /*
      *   label: required label for the graph; used for the aria-label attribute
      
      Defaults to ''.
      */

      label: '',
      /*
      *   describedBy: id to use as the aria-describedby attribute
      
      Defaults to ''.
      */

      describedBy: '',
      /*
      *   percentageDisplayType: determines how to display percentage values(options: decimal, wholeNumber)
      
      Defaults to decimal.
      */

      percentageDisplayType: 'decimal',
      /*
      *   description: description to put in the div for the graph desscription, linked via describedBy
      
      Defaults to ''.
      */

      description: '',
      /*
      *   width: The width of the "graph" area, not of the entire display. Full
      width is calculated from this and margin settings.
      
      Defaults to 620.
      */

      width: 620,
      /*
      *   height: The height of the "graph" area, same caveats as width.
      
      Defaults to 400
      */

      height: 400,
      /*
      *   margin: A dicitionary, keyed 'top', 'right', 'bottom', and 'left', of margin values.
      Left and bottom margins are used to give space to grid marker labels.
      
      Defaults to
          {
            top: 1,
            right: 20,
            bottom: 40,
            left: 60
          }
      */

      margin: {
        top: 40,
        right: 40,
        bottom: 40,
        left: 40
      },
      /*
      *   axes: The axes of the graph. Currently supports xAxis and yAxis. The label, min, max,
      and tickSize can be set for each axis.
      
      Defaults to
          {
            xAxis: {
              label: 'X',
              min: 0,
              max: 0,
              tickSize: 2
            },
            yAxis: {
              label: 'Y',
              min: 0,
              max: 0,
              tickSize: 2
            }
          }
      */

      axes: {
        xAxis: {
          label: 'X',
          min: 0,
          max: 10,
          tickSize: 2
        },
        yAxis: {
          label: 'Y',
          min: 0,
          max: 10,
          tickSize: 2
        }
      },
      /*
      *   legend: A dictionary of key-value strings to define the legend display.
      Each row of the legend will be classed based off the key (in
      graph-legend-[key] format), and contains an empty .graph-legend-icon div
      and a .graph-legend-label div containing the value as content.
      
      Defaults to null.
      
      Example:
          legend: {
            "profit": "YTD Profits",
            "loss": "Gross Losses"
          }
      */

      legend: null,
      /*
      *   tools: An array of tool configurations. Configurations take the following format:
      *   type: The type of the tool, used to load and instantiate the appropriate library.
          See ./tools for a list of supproted tool types. Add your own if neceesary.
      *   data: The config object that is passed to the Tool's constructor.  See the specific
          tool documentation for details.
      */

      tools: [],
      /*
      *   layers: An array of strings representing the ids of the layers you want in the graph.
      Each layer may contain any number of drawn objects, so it is up to the developer to
      manage any restriction rules therein. Layers are presented in the order provided by this
      array, with the first layer being behind any others.
      
      'default' and 'grid' layers are always created, regardless of the contents of this configuration.
      However, if you wish to arrange the order of these and custom layers, simply include
      the 'default' and 'grid' layers in your configuration in the desired order.
      
      Defaults to an empty array.
      
      Example:
          layers: ['background', 'grid', 'preset', 'user']
      */

      layers: [],
      /*
      *   colors: An array of strings representing the ids of colors used by the graph. Note that
      these colors should __not__ be hex or rgba color values. The ids given are simply used
      as class labels, accessible in CSS as .graph-color-[color id]. All visual styling should
      be done in CSS.
      
      A 'default' color is always created, regardless of the contents of this configuration.
      
      Defaults to an empty array.
      
      Example:
          colors: ['demand', 'profit']
      */

      colors: [],
      /*
      *   presets: An array of GraphObject configurations to initialize the graph with. Configurations
      take the following format:
      *   type: The type of the object, used to load and instantiate the appropriate library.
          See ./objects for a list of supported object types. Add your own if necessary.
      *   data: The config object that is passed to the GraphObject's constructor.  See the
          specific object documentation for details.
      *   color: The id of the intended color for the created object, as listed in
          the colors array above.
      *   layer: The id of the intended layer for the created object, as listed in
          the layers array above.
      
      This set of objects is imported via the import() method, and given the id "presets".
      
      Defaults to an empty array.
      
      Example:
          presets: [
            {
              type: 'point',
              data: {
                x: 5,
                y: 2.4
              },
              color: 'default',
              layer: 'preset'
            }
          ]
      */

      presets: []
    };
    return Graph = (function(_super) {
      __extends(Graph, _super);

      function Graph(conf) {
        this.onToolsLoaded = __bind(this.onToolsLoaded, this);
        this.eventPassthrough = __bind(this.eventPassthrough, this);
        this.onActionEnd = __bind(this.onActionEnd, this);
        this.onActionMove = __bind(this.onActionMove, this);
        this.onActionStart = __bind(this.onActionStart, this);
        this.onToolSelectionRequest = __bind(this.onToolSelectionRequest, this);
        this.getPositionOffset = __bind(this.getPositionOffset, this);
        this.getColor = __bind(this.getColor, this);
        this.setDescription = __bind(this.setDescription, this);
        this.setTitle = __bind(this.setTitle, this);
        this.enable = __bind(this.enable, this);
        this.disable = __bind(this.disable, this);
        this.asD3 = __bind(this.asD3, this);
        this.$dom = __bind(this.$dom, this);
        this.addTool = __bind(this.addTool, this);
        this.removeObject = __bind(this.removeObject, this);
        this.addObject = __bind(this.addObject, this);
        this.getObjectsInLayer = __bind(this.getObjectsInLayer, this);
        this._buildObjectFactory = __bind(this._buildObjectFactory, this);
        this["import"] = __bind(this["import"], this);
        this["export"] = __bind(this["export"], this);
        var config;
        config = $.extend(true, {}, defaults, conf);
        this.model = new Model(config);
        this.model.addEventListener('Graph.ObjectAdded', this.eventPassthrough);
        this.model.addEventListener('Graph.ObjectRemoved', this.eventPassthrough);
        this.model.addEventListener('Graph.ToolsLoaded', this.onToolsLoaded);
        this.model.loadTools();
        this._view = new View(config, this.model);
        this._view.addEventListener('Graph.ToolSelectionRequest', this.onToolSelectionRequest);
        this._view.addEventListener('Graph.ActionStart', this.onActionStart);
        this._view.addEventListener('Graph.ActionMove', this.onActionMove);
        this._view.addEventListener('Graph.ActionEnd', this.onActionEnd);
        this._imports = {};
        if (config.presets != null) {
          this["import"](config.presets, 'presets');
        }
        this._active = true;
      }

      /*
      API
      ===
      
      export(layers)
      --------------
      Takes an array of layer ids and exports all of the GraphObjects contained in those layers.
      The exported data is identical in structure to the presets configuration structure.
      
      Returns an array of object configurations.
      */


      Graph.prototype["export"] = function(layers) {
        var exp, l, obj, objExport, _i, _j, _len, _len1, _ref;
        exp = [];
        for (_i = 0, _len = layers.length; _i < _len; _i++) {
          l = layers[_i];
          _ref = this.model.getObjectsInLayer(l);
          for (_j = 0, _len1 = _ref.length; _j < _len1; _j++) {
            obj = _ref[_j];
            objExport = obj["export"]();
            objExport.layer = l;
            exp.push(objExport);
          }
        }
        return exp;
      };

      /*
      import(data, group = null)
      ------------
      The inverse of export, expects an array of object configurations to build.
      Takes an optional group parameter. If none is provided, one is generated from
      the current timestamp. This group id is used as reference for managing
      import completion event listeners.
      
      Returns the group id.
      */


      Graph.prototype["import"] = function(data, group) {
        var importGroup, objData, _i, _len;
        importGroup = group || (new Date()).getTime();
        this._imports[importGroup.toString()] = {
          loaded: 0,
          total: data.length
        };
        for (_i = 0, _len = data.length; _i < _len; _i++) {
          objData = data[_i];
          objData.importGroup = importGroup;
          if (objData.layer == null) {
            objData.layer = 'default';
          }
          require(['./objects/' + objData.type + '/object'], this._buildObjectFactory(objData));
        }
        return importGroup;
      };

      Graph.prototype._buildObjectFactory = function(settings) {
        var _this = this;
        return function(ObjectClass) {
          var imp, obj;
          obj = new ObjectClass(settings.data);
          obj.setColor(_this.model.getColorById(settings.color));
          _this.model.addObjectToLayer(obj, settings.layer);
          imp = _this._imports[settings.importGroup];
          imp.loaded++;
          if (imp.loaded === imp.total) {
            return _this.dispatchEvent('Graph.ImportComplete', {
              group: settings.importGroup
            });
          }
        };
      };

      /*
      getObjectsInLayer(layer)
      ------------------------
      Returns all objects in the given layer.
      */


      Graph.prototype.getObjectsInLayer = function(layer) {
        return this.model.getObjectsInLayer(layer);
      };

      /*
      addObject(obj, layer = null)
      --------------
      Adds the given object to the graph. The provided object is expected to be an instance of
      a subclass of GraphObject.
      
      If a layer (string id) is provided, then the object will be added to the specified layer.
      */


      Graph.prototype.addObject = function(obj, layer) {
        if (layer == null) {
          layer = null;
        }
        if (layer != null) {
          return this.model.addObjectToLayer(obj, layer);
        } else {
          return this.model.addObject(obj);
        }
      };

      /*
      removeObject(obj)
      -----------------
      Removes the given object from the graph.
      */


      Graph.prototype.removeObject = function(obj) {
        return this.model.removeObject(obj);
      };

      /*
      addTool(tool)
      -------------
      Adds the given tool to the toolset. The provided tool is expected to be an
      instance of a subclass of Tool.
      */


      Graph.prototype.addTool = function(tool) {
        return this.model.addTool(tool);
      };

      /*
      $dom()
      ------
      Returns the dom for the drawgraph component for embedding.
      */


      Graph.prototype.$dom = function() {
        return this._view.$dom;
      };

      /*
      asD3()
      */


      Graph.prototype.asD3 = function() {
        return this._view.svg;
      };

      /*
      disable()
      ---------
      Disables user interaction with the graph.
      */


      Graph.prototype.disable = function() {
        this._active = false;
        return this.model.disable();
      };

      /*
      enable()
      --------
      Enables user interaction with the graph.
      */


      Graph.prototype.enable = function() {
        this._active = true;
        return this.model.enable();
      };

      /*
      setTitle(title)
      ---------------
      Updates the title displayed above the graph.
      */


      Graph.prototype.setTitle = function(title) {
        return this._view.updateTitle(title);
      };

      /*
      setDescription(description)
      ---------------
      Updates the graph description for accessibility
      */


      Graph.prototype.setDescription = function(description) {
        return this._view.updateDescription(description);
      };

      Graph.prototype.getColor = function(id) {
        return this.model.getColorById(id);
      };

      /*
      getPositionOffset(position)
      ---------------
      Gets the offset position of the given graph position relative to the graph element
      */


      Graph.prototype.getPositionOffset = function(position) {
        return this._view.getPositionOffset(position);
      };

      /*
      Events
      ======
      The drawgraph component fires the following events:
      
      Graph.ObjectAdded
      -----------------
      Fired when an object has been added to the graph. Event payload has the following format:
      *   newObject: The object that has been added.
      *   layer: The layer to which the object was added.
      
      Graph.ActionComplete
      -------------------
      Fired when a tool has completed an action.
      
      Graph.ObjectRemoved
      -------------------
      Fired when an object has been removed from the graph. Event payload has the following format:
      *   obj: The object that has been removed.
      *   layer: The layer from which the object was added.
      
      Graph.Ready
      -----------
      Fired when all graph tools have been loaded.  This signifies that the graph is ready
      to be used.
      
      Graph.ImportComplete
      --------------------
      Fired when an import has been completed. Event payload has the following format:
      *   group: The id of the import group that triggered the event.
      
      If presets are provided to the constructor, the group id is "presets".
      
      Internal Event Handlers
      =======================
      */


      Graph.prototype.onToolSelectionRequest = function(evt, data) {
        if (this._active) {
          switch (data.tool.type()) {
            case 'drag':
              return this.model.setCurrentTool(data.tool);
            case 'click':
              data.tool.click(this.model);
              return this.dispatchEvent('Graph.ActionComplete', {});
          }
        }
      };

      Graph.prototype.onActionStart = function(evt, data) {
        if (this._active) {
          return this.model.currentTool.start(data.pos, this.model);
        }
      };

      Graph.prototype.onActionMove = function(evt, data) {
        if (this._active) {
          this.model.currentTool.move(data.pos, this.model);
          return this.dispatchEvent('Graph.ActionMove', {});
        }
      };

      Graph.prototype.onActionEnd = function(evt, data) {
        if (this._active) {
          this.model.currentTool.end(this.model);
          return this.dispatchEvent('Graph.ActionComplete', {});
        }
      };

      Graph.prototype.eventPassthrough = function(evt, data) {
        if (this._active) {
          return this.dispatchEvent(evt, data);
        }
      };

      Graph.prototype.onToolsLoaded = function() {
        this.dispatchEvent('Graph.Ready', {});
        if (!this._active) {
          return this.model.disable();
        }
      };

      return Graph;

    })(EventDispatcher);
  });

}).call(this);
