require([
    "domReady",
    "jquery",
    "marionette",
    "sidebar/SidebarView",
    "tep/tep",
    "mathjax",
    "ConceptCodesplits",
    "common/js/app/ssefront",
    "common/js/app/bcore-require",
    "common/js/vendor/hopscotch",
    // TODO - move these into videoUpload so they're not blocking init-concept
    "kaltura/js/vendor/jquery.ui.widget",
    "kaltura/js/jquery.iframe-transport",
    "kaltura/js/jquery.fileupload",
], function(
    domReady,
    $,
    Marionette,
    SidebarView,
    TEP,
    MathJax,
    ConceptCodesplits,
) {

    // Require.js equivalent of jquery dom ready event
    domReady(function () {
        // Tooltip
        $(".tip").tooltip();

        // Window resize with sidebar
        var bodyHeaderElement = $("#conceptBody, .concept-header");
        $(window).resize(function() {
            var utilityPanePosition = $(".utility-pane").position().left;
            if (utilityPanePosition == 50) {
                var windowWidth = $(window).width();
                bodyHeaderElement.width(windowWidth - 350);
            } else {
                var windowWidth = $(window).width();
                bodyHeaderElement.width(windowWidth);
            }
        });

        // Polyfill CustomEvent() constructor functionality in Internet Explorer 9 and higher
        // This is to enable CustomEvents in IE browsers which don't support it (only MS Edge does)
        (function () {
            function CustomEvent ( event, params ) {
            params = params || { bubbles: false, cancelable: false, detail: undefined };
            var evt = document.createEvent( 'CustomEvent' );
            evt.initCustomEvent( event, params.bubbles, params.cancelable, params.detail );
            return evt;
            }
            CustomEvent.prototype = window.Event.prototype;
            window.CustomEvent = CustomEvent;
        })();

    });

    // ONLY load Tour if required so
    var startTour = function() {
        ConceptCodesplits.getTourFactoryPromise().then(function (TourFactory) {
            var tourFactory = new TourFactory();
            var tour = tourFactory.createTour(tourFactory.toursDictionary["core-concept"]);
            // I wasn't able to access hopscotch in this scope without prepending window
            if(window.hopscotch.getState())
                window.hopscotch.endTour();
            window.hopscotch.startTour(tour.tour);
            window.first_time_logged = false;
        });
    };

    $("#tourButton").on("click", function(){
        startTour();
    });

    $("#toolsSaveClose").on("click keyup", function(evt) {
        if (evt.keyCode == 13 || evt.type == "click") {
            $("#toolsSave").modal("hide");
            var anotherModalOpen = false;
            $(document).find(".modal").each(function (e) {
                if (($(this).data("bs.modal") || {isShown: false}).isShown) {
                    anotherModalOpen = true;
                    $(this).find('[tabindex]').first().focus();
                }
            });
            if (anotherModalOpen)
                $("body").addClass("modal-open");
        }
    });

    // Modal
    $(".modal").on("show",function() {
        $("body").addClass("modal-open");
    }).on("hidden", function() {
        $("body").removeClass("modal-open");
    });

    // Sidebar Application, make it global for access related reasons
    window.sidebarApp = new Marionette.Application({
        disabledComponentsCheck: function(component_list) {
            this.rootView.disabledComponentsCheck(component_list);
        }
    });
    sidebarApp.on("before:start", function (options) {
        this.rootView = new SidebarView();
        this.processConceptUrl = function(url) {
            return this.rootView.processConceptUrl(url);
        }
    });
    sidebarApp.on("start", function (options) {
        this.rootView.render();
    });
    sidebarApp.start();

    // Next/Previous Concept arrows depend on the sidebarApp pre-process url
    $(".concept-navigation-link").on("click", function(ev) {
        ev.preventDefault();
        var conceptUrl = $(ev.currentTarget).attr("href");
        if (conceptUrl)
            window.location = sidebarApp.processConceptUrl(conceptUrl);
    });

    // Process titles
    MathJax.Hub.Queue(["Typeset", MathJax.Hub, $('.concept-page-title')[0]]);
    MathJax.Hub.Queue(function() {
        $('.concept-page-title').css('visibility', 'visible');
    });
    MathJax.Hub.Queue(["Typeset", MathJax.Hub, $('.concept-page-subtitle')[0]]);
    MathJax.Hub.Queue(function() {
        $('.concept-page-subtitle').css('visibility', 'visible');
    });
    MathJax.Hub.Process();

    // TEP event handlers
    TEP.addEventListener("TEI.Ready", function(evt, data) {
        // Event kicked when each single TEI gets ready
    });
    TEP.addEventListener("TEP.AllTEIReady", function() {
        // Sending custom event for scripts not loading TEP directly.
        document.dispatchEvent(new Event('TEP.AllTEIReady'));
        // Set IE Specific css for MathJax
        MathJax.Hub.Browser.Select({
            MSIE: function(browser) {
                if (browser.versionAtLeast("8.0"))
                    $('#MathJax_MSIE_Frame').css({'overflow': 'hidden', 'display': 'none'});
            }
        });

        // Add a rerender step to the queue to ensure it is done last
        MathJax.Hub.Queue(["Rerender", MathJax.Hub]);
    });

    // TEP Events available for usage
    // TEP.addEventListener("TEP.GlossaryEnable", function() {});
    // TEP.addEventListener("TEP.GlossaryDisable", function() {});
    // TEP.addEventListener("TEI.Complete", function(evt, data) {});

    TEP.metricsActive = window.globals.metricsActive;

    // Run TEP platform and make it globally accessible
    TEP.run();
    window.TEP = TEP;

    if(window.first_time_logged)
        startTour();

    //Interaction Container
    $('.int-toggle').click(function(event) {
        if ($(event.target).closest('li').is(":not(.disabled)") && $(event.target).closest('li').is(":not(.open)")) {
            $('.int-toggle .open').removeClass('open');
            $('.interaction-container .open').slideUp().removeClass('open');
            $('.interaction-points').addClass('open-children');
            var id = $(event.target).closest('li').addClass('open').attr('id').split('-')[2];
            $(".interaction-container #int-" + id).slideDown().addClass('open');
        }
        else if ($(event.target).closest('li').hasClass("open")) {
            $('.interaction-container .open').slideUp();
            $('.int-toggle .open').removeClass('open');
            $('.interaction-points').removeClass('open-children');
        }
        return false;
    });

    $('.interaction-container .close').click(function() {
        $(this).parent().slideUp().removeClass('open');
        $('.interaction-points').removeClass('open-children');
        return false;
    });

    //Drill Down Content -
    var allPanels = $('.additional-content li .accordian-content').hide();
    $('.additional-content .accordian-toggle').click(function() {
        if ($(this).parent().hasClass('open')) {
            $(this).next().slideUp().parent().removeClass("open");
            return false;
        }
        else {
            allPanels.slideUp().parent().removeClass("open");
            $(this).next().slideDown().parent().addClass("open");
            return false;
        }
    });
});
