/* ===================================================
    HBX Custom Modifications

    Developer: Marinko Peso
 *  Function: jQuery.fn.stickyNotes.changeNotesHeight
    Desc:
    Move all notes currently visible on y axis by value specified.
    Used to reposition in case of height changes.

    Developer: Marinko Peso
 *  Function: jQuery.fn.stickyNotes.exportJSON
    Desc:
    Export all notes as in JSON format for saving as state on the server.

    Developer: Marinko Peso
 *  Function: jQuery.fn.stickyNotes.importJSON
    Desc:
    Import JSON data from the server and recreate notes state based on it.

    Developer: Marinko Peso
 *  Function: jQuery.fn.stickyNotes.deleteAll
    Desc:
    Delete all current notes, both as elements and in memory.
    Used as a part of ClearAll functionality.

    Developer: Marinko Peso
 *  Function: jQuery.fn.stickyNotes.stopEditingAll
    Desc:
    Disable edit mode for all currently edited stickers. This will save their
    current state and prevent loss of data. Used prior of save in HBX Student Tools.

    Developer: Marinko Peso
 *  Option: jQuery.fn.stickyNotes.options.logCallback
    Desc:
    Callback reference to HBX Student Tools function that saves memory state.
    Used to add sticky notes to HBX Student Tools undo functionality.
    Used on multiple places in this file.

    Developer: Marinko Peso
 *  Option: jQuery.fn.stickyNotes.options.style_class
    Desc:
    Additional parameter used to define set of custom styles.
    Used to style notes and save their style in HBX Student Tools.

    Developer: Marinko Peso
 *  Function Modified: jQuery.fn.stickyNotes.editNote
    Desc:
    Added call to autosize() library. Purpose is to automatically resize
    textarea based on amount of text typed in, until the max-height property.
    Also added call to moveCursorToEnd function.

    Developer: Marinko Peso
 *  Function Modified: jQuery.fn.stickyNotes.deleteNote
    Desc:
    Added call to autosize.destroy to unbind textareas that are being removed
    from the page.

    Developer: Marinko Peso
 *  Function: jQuery.fn.stickyNotes.moveCursorToEnd
    Desc:
    Move cursor to the end of currently active textarea. Used when focusing
    sticky notes with already existing content.

    Developer: Ena Livaja
 *  Function Modified: jQuery.fn.stickyNotes.importJSON
    Desc: htmlDecode added.
 * ========================================================== */

;(function( jQuery ) {

    jQuery.fn.stickyNotes = function(options) {
        jQuery.fn.stickyNotes.options = jQuery.extend({}, jQuery.fn.stickyNotes.defaults, options);
        jQuery.fn.stickyNotes.prepareContainer(this);
        jQuery.each(jQuery.fn.stickyNotes.options.notes, function(index, note){
            jQuery.fn.stickyNotes.renderNote(note);
            jQuery.fn.stickyNotes.notes.push(note);
        });
    };

    jQuery.fn.stickyNotes.getNote = function(note_id) {
        var result = null;
        jQuery.each(jQuery.fn.stickyNotes.notes, function(index, note) {
            if (note.id == note_id) {
                result = note;
                return false;
            }
        });
        return result;
    };

    jQuery.fn.stickyNotes.getNotes = function() {
        return jQuery.fn.stickyNotes.notes;
    };

    jQuery.fn.stickyNotes.deleteNoteID =  function(note_id) {
        jQuery.each(jQuery.fn.stickyNotes.notes, function(index, note) {
            if (note.id == note_id) {
                if (jQuery.fn.stickyNotes.options.logCallback) {
                    jQuery.fn.stickyNotes.options.logCallback(jQuery.fn.stickyNotes.options.toolsObject);
                }
                jQuery.fn.stickyNotes.notes.splice(index, 1);
                return false;
            }
        });
    };

    jQuery.fn.stickyNotes.changeNotesHeight = function(diff) {
        jQuery.each(jQuery.fn.stickyNotes.notes, function(index, note) {
            var note_element = $("#note-" + note.id);
            note.pos_y = note.pos_y + diff;
            note_element.css("top", parseInt(note_element.position().top) + diff);
        });
    };

    jQuery.fn.stickyNotes.exportJSON = function() {
        return JSON.stringify(jQuery.fn.stickyNotes.notes);
    };

    jQuery.fn.stickyNotes.importJSON = function(notes) {
        notes = $.parseJSON(notes);
        jQuery.fn.stickyNotes.deleteAll();
        jQuery.each(notes, function(index, note) {
            // hack for single quotes
            note.text = note.text.replace(/&#39;/gi, "'");
            jQuery.fn.stickyNotes.renderNote(note);
            note.text = Bcore.htmlDecode(note.text.replace(/<br\s*[\/]?>/gi, "\n"));
            jQuery.fn.stickyNotes.notes.push(note);
        });
    };

    jQuery.fn.stickyNotes.deleteAll = function() {
        $("div[id^='note-']").remove();
        jQuery.fn.stickyNotes.notes = [];
    };

    jQuery.fn.stickyNotes.prepareContainer = function(container) {
        if (jQuery.fn.stickyNotes.options.controls) {
            jQuery("#sticky-container").html('<button id="add_note">Add Note</button>');
            jQuery("#add_note").click(function() {
                jQuery.fn.stickyNotes.createNote();
                return false;
            });
        }
        jQuery("#sticky-container").click(function() {
            var note_ids = jQuery.fn.stickyNotes.currentlyEditedNotes();
            for (var i = note_ids.length - 1; i >= 0; i--){
                var note_id = note_ids[i];
                if (note_id !== null) {
                    jQuery.fn.stickyNotes.stopEditing(note_id);
                }
            }
        });
    };

    jQuery.fn.stickyNotes.createNote = function(style_class) {
        if (jQuery.fn.stickyNotes.options.logCallback) {
            jQuery.fn.stickyNotes.options.logCallback(jQuery.fn.stickyNotes.options.toolsObject);
        }

        // calculate current user position
        var pos_y = 0;
        var scroll_pos = $(document).scrollTop();
        var container_pos = $("#sticky-container").offset().top;
        if (scroll_pos > container_pos) {
            var current_y =  scroll_pos - container_pos;
            var current_height = $("#sticky-container").height();
            if ((current_y + 240) > current_height)
                pos_y = current_height - 240;
            else
                pos_y = current_y + 60;
        }
        var pos_x = 0;
        var current_width = $("#sticky-container").width();
        pos_x = current_width/2 - 90;

        var note_id = jQuery.fn.stickyNotes.notes.length + 1;
        var _note_content = jQuery(document.createElement('textarea'));

        var _div_note = jQuery(document.createElement('div')).addClass('jStickyNote').addClass(style_class+"StickyText").addClass(style_class+"Sticky");
        _div_note.append(_note_content);
        var _div_delete = $(document.createElement('div'))
                            .addClass('jSticky-delete')
                            .click(function(){jQuery.fn.stickyNotes.deleteNote(this);});
        var _div_wrap = $(document.createElement('div'))
                            .css({'position':'absolute','top':pos_y,'left':pos_x, 'float' : 'left'})
                            .attr("id", "note-" + note_id)
                            .append(_div_note)
                            .append(_div_delete);
        _div_wrap.addClass('jSticky-medium');
        if (jQuery.fn.stickyNotes.options.resizable) {
            _div_wrap.resizable({stop: function(event, ui) { jQuery.fn.stickyNotes.resizedNote(note_id);}});
        }
        _div_wrap.draggable({containment: '#sticky-container', scroll: false, stop: function(event, ui){jQuery.fn.stickyNotes.movedNote(note_id);}});


        $('#sticky-container').append(_div_wrap);

        jQuery.fn.stickyNotes.setCurrentlyEditedNote(note_id);
        jQuery("#note-" + note_id).click(function() {
            return false;
        });
        jQuery("#note-" + note_id).find("textarea").focus().autosize();
        var note = {
            "id":note_id,
            "text":"",
            "pos_x": pos_x,
            "pos_y": pos_y,
            "width": jQuery(_div_wrap).width(),
            "style_class": style_class
        };
        jQuery.fn.stickyNotes.notes.push(note);
        jQuery(_note_content).css({
            'width': jQuery("#note-" + note_id).width() - 25,
            'height': jQuery("#note-" + note_id).height() - 40
        });
        if (jQuery.fn.stickyNotes.options.createCallback) {
            jQuery.fn.stickyNotes.options.createCallback(note);
        }

    };

    jQuery.fn.stickyNotes.stopEditing = function(note_id) {
        var note = jQuery.fn.stickyNotes.getNote(note_id);
        if (note) {
            var new_text = $("#note-" + note_id).find('textarea').val();
            if (note.text != new_text) {
                if (jQuery.fn.stickyNotes.options.logCallback) {
                    jQuery.fn.stickyNotes.options.logCallback(jQuery.fn.stickyNotes.options.toolsObject);
                }
            }
            note.text = new_text;
            // replace editing <textarea> with a new <p> tag with escaped content
            var _p_note_text = $(document.createElement('p')).attr("id", "p-note-" + note_id).html(Bcore.htmlEncode(note.text).replace(/[\n]/g, "<br />"));
            $("#note-" + note_id).find('textarea').replaceWith(_p_note_text);
            //if on device add click event listener for editing,  if not add double click
            if (window.device) {
                    $("#p-note-" + note_id).click(function() {
                    jQuery.fn.stickyNotes.editNote(this);
                    });
            } else {
                    $("#p-note-" + note_id).dblclick(function() {
                    jQuery.fn.stickyNotes.editNote(this);
                    });
             }
        }
        jQuery.fn.stickyNotes.removeCurrentlyEditedNote(note_id);
        if (jQuery.fn.stickyNotes.options.editCallback) {
            jQuery.fn.stickyNotes.options.editCallback(note);
        }

    };

    jQuery.fn.stickyNotes.stopEditingAll = function() {
        var notes = jQuery.fn.stickyNotes.currentlyEditedNotes();
        jQuery.each(notes, function(index, note) {
            jQuery.fn.stickyNotes.stopEditing(note);
        });
    };

    jQuery.fn.stickyNotes.deleteNote = function(delete_button) {
        var note_id = jQuery(delete_button).parent().attr("id").replace(/note-/, "");
        var note = jQuery.fn.stickyNotes.getNote(note_id);
        $('#textarea-note-' + note_id).trigger('autosize.destroy');
        jQuery("#note-" + note_id).remove();
        if (jQuery.fn.stickyNotes.options.deleteCallback) {
            jQuery.fn.stickyNotes.options.deleteCallback(note);
        }
        jQuery.fn.stickyNotes.deleteNoteID(note_id);
    };

    jQuery.fn.stickyNotes.editNote = function(paragraph) {
        var note_id = jQuery(paragraph).parent().parent().attr("id").replace(/note-/, "");
        var content = $("#note-" + note_id).find('p').html();
        content = Bcore.htmlDecode(content.replace(/<br\s*[\/]?>/gi, "\n"));
        var textarea = $(document.createElement('textarea')).attr("id", "textarea-note-" + note_id).val(content);

        $("#p-note-" + note_id).replaceWith(textarea);

        jQuery(textarea).css({
            'width': jQuery("#note-" + note_id).width() - 25
        });
        var current_textarea = jQuery("#note-" + note_id).find("textarea");
        current_textarea.focus().autosize();
        this.moveCursorToEnd(current_textarea.get(0));
        jQuery.fn.stickyNotes.setCurrentlyEditedNote(note_id);
    };

    jQuery.fn.stickyNotes.moveCursorToEnd = function(el) {
        if (typeof el.selectionStart == "number") {
            el.selectionStart = el.selectionEnd = el.value.length;
        } else if (typeof el.createTextRange != "undefined") {
            el.focus();
            var range = el.createTextRange();
            range.collapse(false);
            range.select();
        }
    };

    jQuery.fn.stickyNotes.currentlyEditedNotes = function() {
        return jQuery.fn.stickyNotes.currentlyEditedNoteIds;
    };

    jQuery.fn.stickyNotes.setCurrentlyEditedNote = function(note_id) {
        jQuery.fn.stickyNotes.currentlyEditedNoteIds.push(note_id);
    };

    jQuery.fn.stickyNotes.removeCurrentlyEditedNote = function(note_id) {
        var notes = jQuery.fn.stickyNotes.currentlyEditedNotes();
        var pos = -1;
        for (var i = notes.length - 1; i >= 0; i--){
            if (notes[i] == note_id) {
                pos = i;
                break;
            }
        }
        jQuery.fn.stickyNotes.currentlyEditedNoteIds.splice(pos, 1);
    };

    jQuery.fn.stickyNotes.renderNote = function(note) {
        var _p_note_text = $(document.createElement('p')).attr("id", "p-note-" + note.id).html(note.text);
        var _div_note = jQuery(document.createElement('div')).addClass('jStickyNote').addClass(note.style_class+"StickyText").addClass(note.style_class+"Sticky");
        _div_note.append(_p_note_text);
        var _div_delete = $(document.createElement('div'))
                            .addClass('jSticky-delete')
                            .click(function(){jQuery.fn.stickyNotes.deleteNote(this);});

        var _div_wrap = $(document.createElement('div'))
                            .css({'position':'absolute', 'top':parseInt(note.pos_y), 'left':parseInt(note.pos_x), 'float': 'left',"width":note.width,"height":note.height})
                            .attr("id", "note-" + note.id)
                            .append(_div_note)
                            .append(_div_delete);
        _div_wrap.addClass('jSticky-medium');
        if (jQuery.fn.stickyNotes.options.resizable) {
            _div_wrap.resizable({stop: function(event, ui) { jQuery.fn.stickyNotes.resizedNote(note.id);}});
        }
        _div_wrap.draggable({containment: '#sticky-container', scroll: false, stop: function(event, ui){jQuery.fn.stickyNotes.movedNote(note.id);}});
        $('#sticky-container').append(_div_wrap);
        jQuery("#note-" + note.id).click(function() {
            return false;
        });
        $(_p_note_text).autosize();

        //if on device add click event listener for editing, if not add double click
        if (window.device) {
            $(_p_note_text).click(function() {
            jQuery.fn.stickyNotes.editNote(this);
            });
        } else {
           $(_p_note_text).dblclick(function() {
            jQuery.fn.stickyNotes.editNote(this);
            });
        }
    };

    jQuery.fn.stickyNotes.movedNote = function(note_id) {
        if (jQuery.fn.stickyNotes.options.logCallback) {
            jQuery.fn.stickyNotes.options.logCallback(jQuery.fn.stickyNotes.options.toolsObject);
        }
        var note = jQuery.fn.stickyNotes.getNote(note_id);
        note.pos_x=jQuery("#note-" + note_id).css("left").replace(/px/, "");
        note.pos_y=jQuery("#note-" + note_id).css("top").replace(/px/, "");
        if (jQuery.fn.stickyNotes.options.moveCallback) {
            jQuery.fn.stickyNotes.options.moveCallback(note);
        }
    };

    jQuery.fn.stickyNotes.resizedNote = function(note_id) {
        var note = jQuery.fn.stickyNotes.getNote(note_id);
        note.width=jQuery("#note-" + note_id).width();
        note.height=jQuery("#note-" + note_id).height();
        if (jQuery.fn.stickyNotes.options.resizeCallback) {
            jQuery.fn.stickyNotes.options.resizeCallback(note);
        }
    };
    jQuery.fn.stickyNotes.createNoteBackground = function(style_class) {
        background = $(document.createElement('div')).addClass("background").html('<div class="stretch backgroundSticky ' + style_class + 'Sticky" style="margin-top:5px;"></div>');
        return background;
    };


    jQuery.fn.stickyNotes.defaults = {
        notes : [],
        resizable : false,
        controls: false,
        bg_image: 'images/sticky-bg.png',
        editCallback: false,
        createCallback: false,
        deleteCallback: false,
        moveCallback: false,
        resizeCallback: false,
        logCallback: false,
        style_class: "black"
    };

    jQuery.fn.stickyNotes.options = null;
    jQuery.fn.stickyNotes.currentlyEditedNoteIds = [];
    jQuery.fn.stickyNotes.notes = [];
})(jQuery);
