// Generated by CoffeeScript 1.6.3
(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  define(function(require) {
    var $, EventDispatcher, Timer, Utils, defaults, hide_btn, tmpl;
    $ = require('jquery');
    EventDispatcher = require('tep/utils/event/event_dispatcher');
    Utils = require('tep/utils/utils');
    require('link!./style.css');
    tmpl = "<div class=\"component-timer\">\n  <img alt=\"\" src=\"\" class=\"stopwatch\"/>\n  <div class=\"timer-time\"></div>\n  <div class=\"screen-reader-visible\" role=\"status\" aria-live=\"polite\"></div>\n</div>";
    hide_btn = '<button type="button"><img src="" alt="Hide timer" /></button>';
    /*
    Timer Component
    ===============
    
    The Timer provides a standard timer for Teaching Elements. It can be configured
    to count up or down, and to fire events at specified intervals.
    
    Configuration
    =============
    Timer supports the following configurations, passed in as a single config object.
    */

    defaults = {
      /*
      *   rate: Sets the direction and speed of the timer in seconds per second.
      For example, a value of 1 makes the timer count up at the normal rate,
      and a value of -2 makes it count down at double-speed.
      For most cases, values of 1 and -1 are expected.
      
      Defaults to 1.
      */

      rate: 1,
      /*
      *   start: Sets the initial time on the clock.  Value is assumed to be in seconds.
      
      Defaults to 0.
      */

      start: 0,
      /*
      *   precision: Sets the granularity of the timer. Higher tps leads
      to higher precision, but is more processor-heavy.
      
      Defaults to 60.
      */

      precision: 60,
      /*
      *   tickRate: Sets how frequently the tick event is fired, in seconds. A value
      of 1 fires once per second, a value of 0.5 fires every half-second.
      
      Defaults to 1.
      */

      tickRate: 1,
      /*
      *   hideable: Determines if users can hide the timer.
      
      Defaults to false.
      */

      hideable: false,
      /*
      *   forceShow: Determines when the timer must be shown if hideable is true.
      Time is in seconds.
      
      Defaults to 300 seconds (5 minutes).
      */

      forceShow: 300,
      /*
      *   classes: Determines classes to add to the timer.
      Can include right, top, bottom, left to change border radius style.
      
      Defaults to ''.
      */

      classes: '',
      /*
      *   timeFirst: Determines if the time should appear on top of the icon.
      
      Defaults to false.
      */

      timeFirst: false
    };
    return Timer = (function(_super) {
      __extends(Timer, _super);

      function Timer(conf) {
        this._render = __bind(this._render, this);
        this._tick = __bind(this._tick, this);
        this.setTime = __bind(this.setTime, this);
        this.getTime = __bind(this.getTime, this);
        this.reset = __bind(this.reset, this);
        this.pause = __bind(this.pause, this);
        this.start = __bind(this.start, this);
        this.slideTimer = __bind(this.slideTimer, this);
        this.config = $.extend({}, defaults, conf);
        this.$dom = $(tmpl).addClass(this.config.classes);
        /*
          append windows variables for icons on img attr src since template can't render
        */

        this.$dom.find(".stopwatch").attr("src", window.stopwatch);
        if (this.config.hideable) {
          this.hide_btn = $(hide_btn);
          this.$dom.prepend(this.hide_btn).addClass('hideable');
          this.hide_btn.on('click', this.slideTimer);
          this.$dom.attr("aria-expanded", "true");
          this.$dom.find("button img").attr("src", window.minus_white);
        }
        if (this.config.timeFirst) {
          this.$dom.prepend(this.$dom.find(".timer-time"));
          this.$dom.addClass("time-first");
        }
        this.reset();
      }

      /*
      API
      ==========
      
      slideTimer()
      -------
      Shows or hides the timer based on if it's currently visible or hidden
      */


      Timer.prototype.slideTimer = function() {
        var $timeDom,
          _this = this;
        $timeDom = this.$dom.find('.timer-time');
        return $timeDom.slideToggle(250, function() {
          if ($timeDom.is(":visible")) {
            _this.$dom.attr("aria-expanded", "true");
            return _this.$dom.find("button img").attr({
              "src": window.minus_white,
              "alt": "Hide timer"
            });
          } else {
            _this.$dom.attr("aria-expanded", "false");
            return _this.$dom.find("button img").attr({
              "src": window.plus_white,
              "alt": "Show timer"
            });
          }
        });
      };

      /*
      start()
      -------
      Starts the timer.
      */


      Timer.prototype.start = function() {
        this._lastTime = Date.now();
        return this._timer = setInterval(this._tick, 1000 / this.config.precision);
      };

      /*
      pause()
      -------
      Pauses the timer. Call start() again to resume.
      */


      Timer.prototype.pause = function() {
        clearInterval(this._timer);
        return this._timer = null;
      };

      /*
      reset()
      -------
      Stops and resets the timer to the initial configuration's start time.
      */


      Timer.prototype.reset = function() {
        this.time = this.config.start * 1000;
        this._lastTick = this.time;
        this.pause();
        return this._render();
      };

      /*
      getTime()
      ---------
      Returns a float of the current time, in seconds.
      */


      Timer.prototype.getTime = function() {
        return this.time / 1000;
      };

      /*
      setTime(time)
      -------------
      Explicitly set the current time, provided in seconds.
      */


      Timer.prototype.setTime = function(time) {
        this.time = time * 1000;
        return this._render();
      };

      /*
      Events
      ======
      Timer fires the following events:
      
      Timer.Tick
      ----------
      Fired when the timer has passed an interval specified by the tickRate configuration.
      Event payload has the following format:
      
      * time: the current time on the timer.
      
      Timer.End
      ---------
      Fired when the timer has reached 0, assuming that the timer counts down.
      
      Private Methods
      ===============
      */


      Timer.prototype._tick = function() {
        var currTime, delta;
        currTime = Date.now();
        delta = currTime - this._lastTime;
        this.time = Math.max(0, this.time + delta * this.config.rate);
        this._render();
        this._lastTime = currTime;
        if (this.time === 0) {
          this.pause();
          return this.dispatchEvent('Timer.End', {});
        } else if (Math.abs(this.time - this._lastTick) > this.config.tickRate * 1000) {
          this.dispatchEvent('Timer.Tick', {
            time: this.getTime()
          });
          this._lastTick += this.config.tickRate * 1000 * this.config.rate;
          if (this.config.hideable && parseInt(this.getTime()) <= parseInt(this.config.forceShow)) {
            if (!this.$dom.find('.timer-time').is(":visible")) {
              this.slideTimer();
            }
            return this.$dom.removeClass('hideable').find('button').remove();
          }
        }
      };

      Timer.prototype._render = function() {
        var accText, t;
        this.$dom.find('.timer-time').text(Utils.timeToString(this.time));
        t = parseInt(this.getTime());
        if (t === 0) {
          accText = "Time limit has been reached.";
        } else if ((t >= 30 && t % 30 === 0) || (t < 30 && t % 10 === 0)) {
          accText = "" + t + " seconds remaining.";
        }
        return this.$dom.find('.timer-text').text(accText);
      };

      return Timer;

    })(EventDispatcher);
  });

}).call(this);
