// Generated by CoffeeScript 1.6.3
(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  define(function() {
    /*
    Math Utilities
    ==============
    Singleton object that provides some common math functionalities.
    */

    var MathUtils;
    MathUtils = (function() {
      function MathUtils() {
        this.distance = __bind(this.distance, this);
        this.posMod = __bind(this.posMod, this);
      }

      /*
      
      radiansToDegrees(radians)
      -------------------------
      Converts a radian value to the same value in degrees.
      */


      MathUtils.prototype.radiansToDegrees = function(r) {
        return (r / Math.PI) * 180;
      };

      /*
      
      degreesToRadians(degrees)
      -------------------------
      Converts a degree value to the same value in radians.
      */


      MathUtils.prototype.degreesToRadians = function(d) {
        return (d / 180) * Math.PI;
      };

      /*
      
      posMod(value, modulo)
      ---------------------
      Provides a "better" modulo operator, which always returns a positive value.
      
      The issue this is trying to solve is that, for exmaple, -1 % 3 in most 
      programming languages evaluates as -1, whereas typically in mathematics, it 
      evaluates as 2. In the modulo-3 space, -1 and 2 are equal (just as 0 and 3 are 
      equal mod 3), but in many practical use cases (e.g. cycling through an array by
      iterating an index up or down modulo the length) -1 and 2 yield very different 
      results, as -1 will throw an out of bounds exception.
      */


      MathUtils.prototype.posMod = function(a, b) {
        return ((a % b) + b) % b;
      };

      /*
      
      distance(a, b)
      --------------
      Provides the Euclidean distance between two points. The points provided
      must be objects with x and y attributes.
      */


      MathUtils.prototype.distance = function(a, b) {
        return Math.sqrt(Math.pow(a.x - b.x, 2) + Math.pow(a.y - b.y, 2));
      };

      return MathUtils;

    })();
    return new MathUtils;
  });

}).call(this);
