// Generated by CoffeeScript 1.6.3
(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  define(function(require) {
    var $, ConceptCodesplits, EventDispatcher, TEProxy, Utils;
    EventDispatcher = require('tep/utils/event/event_dispatcher');
    $ = require('jquery');
    Utils = require('tep/utils/utils');
    ConceptCodesplits = require('ConceptCodesplits');
    /*
    Teaching Element Proxy / Teaching Element Loader
    ================================================
    The Proxy class acts as a temporary stand-in for a TEI while the requisite
    files are loaded and the instance prepares itself for use. As such,
    the Proxy is also the object responsible for managing said loading and
    instantiation.
    */

    return TEProxy = (function(_super) {
      __extends(TEProxy, _super);

      /*
      constructor(domElem, id, autoload = true)
      -----------------------------------------
      Takes the target dom element and TEI ID, and begins the loading process.
      Returns a json document similar to:
      {
          "domElem": {},
          "id": "543e8df0241b5a35be6c56e7",
          "_loadStateOnly": true,
          "$el": {
              "0": {},
              "context": {},
              "length": 1
          }
      }
      If you wish to defer loading, pass in false for autoload, and then call load()
      as needed.
      */


      function TEProxy(domElem, id, autoload, _loadStateOnly) {
        this.domElem = domElem;
        this.id = id;
        if (autoload == null) {
          autoload = true;
        }
        this._loadStateOnly = _loadStateOnly != null ? _loadStateOnly : false;
        this.setView = __bind(this.setView, this);
        this.view = __bind(this.view, this);
        this.conceptFinaleAdjust = __bind(this.conceptFinaleAdjust, this);
        this.onTEReady = __bind(this.onTEReady, this);
        this.onRequirementsLoad = __bind(this.onRequirementsLoad, this);
        this.initTEI = __bind(this.initTEI, this);
        this.onDataLoad = __bind(this.onDataLoad, this);
        this.setAdditionalTeiArguments = __bind(this.setAdditionalTeiArguments, this);
        this.silence = __bind(this.silence, this);
        this.load = __bind(this.load, this);
        this.setParent = __bind(this.setParent, this);
        this.$el = $(this.domElem);
        this.additionalTeiArguments = {};
        if (autoload) {
          this.load();
        }
      }

      /*
      setParent(parent)
      -----------------
      Proxy method for the TEI. If the TEI is loaded, it passes the call on.
      Otherwise, it stores the parent and sets it on the TEI once it has loaded.
      */


      TEProxy.prototype.setParent = function(par) {
        if (this.te) {
          return this.te.setParent(par);
        } else {
          this.parentSet = true;
          return this.parent = par;
        }
      };

      /*
      load()
      ------
      Initiates the loading process.
      TEI loading and instantiation takes the following path:
      *   Make an AJAX call to the TEI api endpoint to grab the input payload.
      */


      TEProxy.prototype.load = function() {
        if (Utils.isGUID(this.id) && !this.te) {
          return $.ajax({
            url: Utils.normalizeUrl('/tei/' + this.id),
            dataType: 'json',
            success: this.onDataLoad
          });
        }
      };

      TEProxy.prototype.silence = function() {
        return this._silent = true;
      };

      TEProxy.prototype.setAdditionalTeiArguments = function(additionalTeiArguments) {
        if (additionalTeiArguments == null) {
          additionalTeiArguments = {};
        }
        return this.additionalTeiArguments = additionalTeiArguments;
      };

      /*
      *   Once the input payload is loaded, use RequireJS to load the appropriate
      main file, based on the TE and version necessary.
      */


      TEProxy.prototype.onDataLoad = function(data, status, xhr) {
        this.sourceData = data;
        this.dispatchEvent("TEProxy.DataLoaded", {
          data: this.sourceData
        });
        if (!this._loadStateOnly) {
          return this.initTEI();
        }
      };

      /*
      Require or includes the code for a TE.
      When using webpack, use import() to generate a separate bundle:
        - https://webpack.js.org/guides/code-splitting/#dynamic-imports
      When on a the original concept page using RequireJS, use require() to dynamically include.
      */


      TEProxy.prototype.initTEI = function() {
        return ConceptCodesplits.getTeachingElementPromise(this.sourceData.te, this.sourceData.version).then(this.onRequirementsLoad);
      };

      TEProxy.prototype.onRequirementsLoad = function(te) {
        this._view = $("<div>").addClass('tei-loading-msg').text('Loading data...');
        this.setView(this._view);
        this.sourceData.input.id = this.sourceData.id;
        this.sourceData.input.te = this.sourceData.te;
        this.sourceData.input.last_tei = this.last_tei;
        if (this.sourceData.sections != null) {
          if (this.sourceData.sections.length !== 0) {
            this.sourceData.input.introduction_title = this.sourceData.introduction_title;
            this.sourceData.input.introduction_title_type = this.sourceData.introduction_title_type;
            this.sourceData.input.introduction_content = this.sourceData.introduction_content;
            this.sourceData.input.interactive_title = this.sourceData.interactive_title;
            this.sourceData.input.interactive_title_type = this.sourceData.interactive_title_type;
            this.sourceData.input.interactive_content = this.sourceData.interactive_content;
            this.sourceData.input.interactive_help = this.sourceData.interactive_help;
            this.sourceData.input.conclusion_title = this.sourceData.conclusion_title;
            this.sourceData.input.conclusion_title_type = this.sourceData.conclusion_title_type;
            this.sourceData.input.conclusion_content = this.sourceData.conclusion_content;
            this.sourceData.input.sections = this.sourceData.sections;
          }
        }
        /*
        *   Once the TE files have been loaded, instantiate the TE using the input
        payload, loading DropBars and TriggerPoints as needed.
        */

        this.te = new te($.extend(this.sourceData.input, this.additionalTeiArguments));
        if (this._silent) {
          this.te.silence();
        }
        if (this.sourceData.drop_bars != null) {
          this.te.loadDropBars(this.sourceData.drop_bars);
        }
        if (this.sourceData.trigger_points != null) {
          this.te.loadTriggerPoints(this.sourceData.trigger_points);
        }
        if (this.sourceData.tei_group != null) {
          this.te.loadGroup(this.sourceData.tei_group);
        }
        this.te.moveButtonBay('end');
        this.te.glossary_enabled = this.sourceData.glossary_enabled;
        /*
        *   Pass the initial state to the newly-created TEI.
        */

        if (this.sourceData.states) {
          this.te.parseState(this.sourceData);
        }
        /*
        *   Set the parent, if appropriate.
        */

        if (this.parentSet) {
          this.te.setParent(this.parent);
        }
        /*
        *   Listen for the ready event, and put the TEI into the dom, hidden from view.
        */

        if (this._finaleType) {
          this.te.conceptFinaleAdjust(this._finaleType);
        }
        this.te.on('ready', this.onTEReady);
        this.$el.append(this.te.view());
        this.te.view().css({
          height: 0,
          overflow: 'hidden'
        });
        return this.te.onScreen();
      };

      /*
      *   Once the TEI is ready, lock it by default, reveal the TEI view, and dispatch the ready event.
      */


      TEProxy.prototype.onTEReady = function(evt, data) {
        var _this = this;
        data.currentTarget.lock();
        return this._view.fadeOut(500, function() {
          _this.te.view().css({
            height: '',
            overflow: '',
            display: 'none'
          });
          return _this.te.view().fadeIn(500, function() {
            _this._view.remove();
            return _this.dispatchEvent('ready', _this);
          });
        });
      };

      TEProxy.prototype.conceptFinaleAdjust = function(_finaleType) {
        if (this.te != null) {
          return this.te.conceptFinaleAdjust(_finaleType);
        } else {
          return this._finaleType = _finaleType;
        }
      };

      /*
      view()
      ------
      Provides the dom element of the proxy (as a jQuery object)
      */


      TEProxy.prototype.view = function() {
        return this.$el;
      };

      /*
      setView(view)
      -------------
      Swaps the current displayed content with the provided content.
      */


      TEProxy.prototype.setView = function(view) {
        return this.$el.html('').append(view);
      };

      return TEProxy;

    })(EventDispatcher);
  });

}).call(this);
