// Generated by CoffeeScript 1.6.3
(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  define(function(require) {
    var $, Question, TeachingElement, attemptsHtml, helpTextHtml, summaryHtml;
    TeachingElement = require('./teaching_element');
    $ = require('jquery');
    summaryHtml = "<div class=\"te-summary\" role=\"status\" aria-live=\"polite\">\n  <h3 class=\"screen-reader-visible\" aria-hidden=\"true\">Result</h3>\n  <p class=\"te-summary-label\" aria-hidden=\"true\"></p>\n  <p class=\"te-grant-credit\" aria-hidden=\"true\">\n    <span class=\"te-grant-credit-label\">(!) You have received credit for this question</span>\n    <br />\n    <span class=\"te-grant-credit-desc\"></span>\n  </p>\n  <p class=\"te-summary-explanation\" aria-hidden=\"true\"></p>\n</div>";
    attemptsHtml = "<div class=\"te-attempt-count-wrap\">\n  <span class=\"te-attempt-label\">Attempts left</span>: <span class=\"te-attempt-count\"></span>\n</div>";
    helpTextHtml = "<div class=\"te-css-accordion\">\n    <ul>\n        <li>\n            <input class=\"help-title\" type=\"checkbox\" checked>\n            <i></i>\n            <h2 class=\"helpTitle\"></h2>\n            <p class=\"helpContent\"></p>\n        </li>\n    </ul>\n</div>";
    /*
    Question
    ========
    Question is the base class for question_provider TeachingElements (TEs that
    are intended for use within a QuestionSet or other question_container TEI).
    
    All Questions share many commonalities that are handled by this class to ease
    development. In particular, they all share the same answer submission process,
    assessment/summary handling, and lifecycle flow. On the visual end, all
    Questions display a bay of buttons and an attempt counter (if more than one
    attempt is allowed).
    */

    return Question = (function(_super) {
      __extends(Question, _super);

      function Question(data, tmpl) {
        this.data = data;
        this.updateQuestionButtonMessage = __bind(this.updateQuestionButtonMessage, this);
        this.addQuestionButtonMessageContainer = __bind(this.addQuestionButtonMessageContainer, this);
        this.checkboxClick = __bind(this.checkboxClick, this);
        this.addHelpTextFields = __bind(this.addHelpTextFields, this);
        this.hideSummary = __bind(this.hideSummary, this);
        this.displayFeedback = __bind(this.displayFeedback, this);
        this.displayIsCorrect = __bind(this.displayIsCorrect, this);
        this.displayCredit = __bind(this.displayCredit, this);
        this.displaySummary = __bind(this.displaySummary, this);
        this.updateAttemptsDisplay = __bind(this.updateAttemptsDisplay, this);
        this.parseState = __bind(this.parseState, this);
        this.onEndRequest = __bind(this.onEndRequest, this);
        this.onSubmitRequest = __bind(this.onSubmitRequest, this);
        this.onStatusChange = __bind(this.onStatusChange, this);
        this.setSkipSummary = __bind(this.setSkipSummary, this);
        this.setAutoComplete = __bind(this.setAutoComplete, this);
        this.setButtonLabel = __bind(this.setButtonLabel, this);
        this.buildButtons = __bind(this.buildButtons, this);
        this.disable = __bind(this.disable, this);
        this.enable = __bind(this.enable, this);
        this.enableButton = __bind(this.enableButton, this);
        this.disableButton = __bind(this.disableButton, this);
        this.hideButton = __bind(this.hideButton, this);
        this.showButton = __bind(this.showButton, this);
        this.submit = __bind(this.submit, this);
        Question.__super__.constructor.call(this, this.data, tmpl);
        this.attemptsDisplay = $(attemptsHtml);
        this.helpTextHtml = $(helpTextHtml);
        this.buttonBay = $('<div>').addClass('question-button-bay');
        this.attemptsAllowed = this.data.attemptsAllowed;
        if (!this.data.hideAttemptsText && this.attemptsAllowed !== -1) {
          this.buttonBay.append(this.attemptsDisplay);
        }
        this.summaryDisplay = $(summaryHtml);
        this.submitCount = 0;
        this.updateAttemptsDisplay();
        this.view().find('.te-question').append(this.summaryDisplay);
        if (this.data.helpTitle) {
          this.addHelpTextFields();
        }
        this.view().find('.te-question').append(this.buttonBay);
        this.buildButtons();
        this.showButton('save');
        this.disableButton('save');
        this.showButton('submit');
        this.disableButton('submit');
        this.on('statuschange', this.onStatusChange);
        this._autocomplete = this.data.autocomplete;
        this._skipSummary = this.data.skipSummary;
        this._skipFeedback = this.data.skipFeedback;
        this.firstLoad = false;
        if (this.data.title != null) {
          this.view().find('.te-title').html(this.data.title);
        } else {
          this.view().find('.te-title').remove();
        }
        if (this.data.preamble != null) {
          this.view().find('.te-preamble').html(this.data.preamble);
        }
        this.$el.find('.helpTitle').text(this.data.helpTitle);
        this.$el.find('.helpContent').html(this.data.helpContent);
        this.content_open = false;
        this.$el.find('.help-title').on('click', this.checkboxClick);
        this.addQuestionButtonMessageContainer();
      }

      /*
      API
      ===
      
      submit()
      --------
      Callback for the submit button. Can also be invoked in other situations as
      necessary. Calls disable() and pushes a user state with the submit operation,
      with a pullState() call in the callback after saving the current state (if save
      buttons are removed and state saves automatically, the outer push state can be removed)
      */


      Question.prototype.submit = function() {
        var _this = this;
        this.disable();
        return this.pushState('user', 'submit', function() {
          return _this.pullState();
        });
      };

      /*
      
      showButton(label)
      -----------------
      Adds the specified button to the button bay and binds the appropriate event
      listeners. See buildButtons() for details about labels.
      */


      Question.prototype.showButton = function(label) {
        var cb, k, _ref, _results;
        if (this.buttons[label] != null) {
          this.buttonBay.append(this.buttons[label].el);
          this.enableButton(label);
          this.buttons[label].el.off('click', this.updateQuestionButtonMessage);
          this.buttons[label].el.on('click', this.updateQuestionButtonMessage);
          _ref = this.buttons[label].events;
          _results = [];
          for (k in _ref) {
            cb = _ref[k];
            this.buttons[label].el.off(k, cb);
            _results.push(this.buttons[label].el.on(k, cb));
          }
          return _results;
        }
      };

      /*
      
      hideButton(label)
      -----------------
      Removes the specified button to the button bay. See buildButtons() for details
      about labels.
      */


      Question.prototype.hideButton = function(label) {
        var btn, k, _ref, _results;
        if (label != null) {
          if (label in this.buttons) {
            return this.buttons[label].el.remove();
          }
        } else {
          _ref = this.buttons;
          _results = [];
          for (k in _ref) {
            btn = _ref[k];
            _results.push(btn.el.remove());
          }
          return _results;
        }
      };

      /*
      
      disableButton(label)
      --------------------
      Disables the button specified by the provided label.
      */


      Question.prototype.disableButton = function(label) {
        if (label in this.buttons) {
          return this.buttons[label].el.attr('disabled', 'disabled');
        }
      };

      /*
      
      enableButton(label)
      -------------------
      Enables the button specified by the provided label.
      */


      Question.prototype.enableButton = function(label) {
        if (label in this.buttons) {
          return this.buttons[label].el.removeAttr('disabled');
        }
      };

      /*
      
      enable()
      --------
      Intended for override by individual TEs. This method is called whenever
      the TEI should be made active and responsive to user input.
      
      Does nothing by default.
      */


      Question.prototype.enable = function() {
        this.isDisabled = false;
        return this.enableButton('continue');
      };

      /*
      
      disable()
      --------
      Intended for override by individual TEs. This method is called whenever
      the TEI should be made inactive.
      
      By default, disables the continue button and disables the submit button to prevent
      repeat submissions.
      */


      Question.prototype.disable = function() {
        this.isDisabled = true;
        this.disableButton('continue');
        this.updateSaveButton(false);
        return this.disableButton('submit');
      };

      /*
      buildButtons()
      --------------
      Called internally, but intended for override by individual TEs as necessary.
      This method populates the this.buttons attribute with a dictionary of button
      definitions. Each button definition is keyed by its label and contains two
      attributes:
      
      *    __el__: the dom element for the button (typically a jQuery object, but not
      necessarily so)
      *    __events__: a dictionary of callbacks keyed by the event string
      
      By default, this method generates two buttons: submit and continue.
      */


      Question.prototype.buildButtons = function() {
        return this.buttons = {
          save: {
            el: $("<button>").attr({
              "class": "btn",
              id: "question-button-save-" + this.id
            }).text('Save'),
            events: {
              click: this.onSaveRequest
            },
            statusMessage: 'Your information has been saved.'
          },
          submit: {
            el: $("<button>").attr({
              "class": "alt-btn",
              id: "question-button-submit-" + this.id
            }).text('Submit'),
            events: {
              click: this.onSubmitRequest
            },
            statusMessage: 'Your information has been submitted.'
          },
          "continue": {
            el: $("<button>").attr({
              "class": "btn",
              id: "question-button-continue-" + this.id
            }).text('Next Question'),
            events: {
              click: this.onEndRequest
            },
            statusMessage: "You continued to the next question."
          }
        };
      };

      Question.prototype.buttonBayBuildButtons = function() {
        return null;
      };

      Question.prototype.setButtonLabel = function(btn, label) {
        this.buttons[btn].el.text(label);
        if (btn === 'continue') {
          return this.buttons[btn].statusMessage = "You continued to the " + (label.toLowerCase()) + ".";
        }
      };

      /*
      setAutoComplete(auto)
      ---------------------
      Used by the QuestionSet to force questions to bypass their own continue button.
      */


      Question.prototype.setAutoComplete = function(_autocomplete) {
        this._autocomplete = _autocomplete;
      };

      /*
      setSkipSummary(skip)
      ---------------------
      Used by the QuestionSet to force questions to skip showing the summary.
      */


      Question.prototype.setSkipSummary = function(_skipSummary) {
        this._skipSummary = _skipSummary;
      };

      /*
      Internal Event Handlers
      =======================
      
      onStatusChange()
      ----------------
      Handles common reactions to status changes. In specific, all Questions by
      default call this.start() as soon as they are unlocked, and call this.disable()
      and this.hideButtons() when complete.
      */


      Question.prototype.onStatusChange = function() {
        if (this.status === 'unlocked') {
          this.start();
          this.firstLoad = true;
        }
        if (this.status === 'complete') {
          this.firstLoad = false;
          this.disable();
          return this.hideButton();
        }
      };

      /*
      
      onSubmitRequest()
      -----------------
      Handles clicks to the submit button by calling this.submit()
      */


      Question.prototype.onSubmitRequest = function() {
        return this.submit();
      };

      /*
      
      onEndRequest()
      --------------
      Handles clicks on the continue button by calling this.end()
      */


      Question.prototype.onEndRequest = function() {
        return this.end();
      };

      /*
      State Handling
      ==============
      
      parseState(data)
      ----------------
      Handles common state parsing, including managing the attempts count display,
      assessment/summary display, and associated button management. Individual
      question TEs will need to manage their own specific state handling outside
      of these considerations.
      */


      Question.prototype.parseState = function(data) {
        var scope, state, _ref, _ref1;
        Question.__super__.parseState.call(this, data);
        _ref = data.states;
        for (scope in _ref) {
          state = _ref[scope];
          switch (scope) {
            case 'user':
              if (state.submitCount != null) {
                this.submitCount = state.submitCount;
                this.updateAttemptsDisplay();
              }
              if ((state.userCorrect != null) && !this._skipSummary) {
                this.displaySummary(state);
              }
              if (state.allowRetry != null) {
                if (state.allowRetry) {
                  this.enable();
                  this.dispatchEvent('retry', this);
                } else {
                  this.disable();
                  this.attemptsDisplay.remove();
                  this.hideButton('save');
                  this.hideButton('submit');
                  if (!this._autocomplete && ((this.status === 'in-progress') || ((state.userCorrect != null) && this.status !== "complete"))) {
                    this.showButton('continue');
                    this.moveButtonBay('end');
                  } else {
                    this.end();
                  }
                }
              } else {
                this.enable();
              }
          }
        }
        if (((_ref1 = data.states) != null ? _ref1.user : void 0) == null) {
          return this.enable();
        }
      };

      Question.prototype.updateAttemptsDisplay = function() {
        return this.attemptsDisplay.find('.te-attempt-count').text(Math.max(0, parseInt(this.attemptsAllowed) - parseInt(this.submitCount)));
      };

      /*
      
      buildSummary(userState)
      -----------------------
      Method for override. Should generate the html or dom element for the summary,
      given the user state object.
      */


      Question.prototype.buildSummary = function(userState) {
        return "";
      };

      /*
      
      displaySummary(userState)
      -------------------------
      Generates and displays the assessment/summary after an answer submission.
      
      See also buildSummary()
      
      Depending on the settings of the question_container, this may or may not get
      called.
      */


      Question.prototype.displaySummary = function(userState) {
        var _this = this;
        this.summaryDisplay.hide();
        this.displayCredit(userState);
        this.displayIsCorrect(userState);
        if (!this._skipFeedback) {
          this.displayFeedback(userState);
        }
        return this.summaryDisplay.fadeIn(250, 'linear', function() {
          if (_this.summaryDisplay.find('.te-summary-explanation').html().indexOf("$$") >= 0 || _this.summaryDisplay.find('.te-summary-explanation').html().indexOf("$!") >= 0) {
            return _this.addLaTeX(_this.summaryDisplay.find('.te-summary-explanation')[0]);
          }
        });
      };

      Question.prototype.displayCredit = function(userState) {
        if (userState.question_credit && !userState.userCorrect && !userState.allowRetry) {
          this.summaryDisplay.find('.te-grant-credit').show();
          return this.summaryDisplay.find('.te-grant-credit-desc').text(userState.question_credit_desc);
        }
      };

      Question.prototype.displayIsCorrect = function(userState) {
        this.summaryDisplay.removeClass('te-summary-correct te-summary-incorrect').addClass(userState.userCorrect ? "te-summary-correct" : "te-summary-incorrect");
        this.summaryDisplay.find('h3').attr('aria-hidden', 'false');
        this.summaryDisplay.find('p').attr('aria-hidden', 'false');
        return this.summaryDisplay.find('.te-summary-label').text(userState.userCorrect ? "Correct!" : "Incorrect" + ((userState.allowRetry != null) && userState.allowRetry ? ", try again" : "") + ".");
      };

      Question.prototype.displayFeedback = function(userState) {
        var explanation;
        explanation = this.buildSummary(userState);
        this.summaryDisplay.find('.te-summary-explanation').html(explanation);
        if (userState.summary != null) {
          if (userState.summary.overview != null) {
            return this.summaryDisplay.find('.te-summary-explanation').append(userState.summary.overview);
          } else if (userState.summary.length > 0 && (userState.summary.filter(function(x) {
            return x.correct == null;
          })).length > 0) {
            return this.summaryDisplay.find('.te-summary-explanation').append(userState.summary[userState.summary.length - 1].content);
          }
        }
      };

      /*
      
      hideSummary()
      -------------
      Removes the assessment/summary display.
      */


      Question.prototype.hideSummary = function() {
        this.summaryDisplay.fadeOut(250, 'linear');
        this.summaryDisplay.find('h3').attr('aria-hidden', 'true');
        return this.summaryDisplay.find('p').attr('aria-hidden', 'true');
      };

      Question.prototype.addHelpTextFields = function() {
        return this.view().find('.te-question').append(this.helpTextHtml);
      };

      Question.prototype.checkboxClick = function() {
        var metricValue;
        this.content_open = !this.content_open;
        metricValue = this.content_open ? {
          question_help: 'open'
        } : {
          question_help: 'closed'
        };
        return this.pushMetric(metricValue, 'interface');
      };

      /*
      
      addQuestionButtonMessageContainer()
      -------------
      Adds accessibility message container to question button bay.
      */


      Question.prototype.addQuestionButtonMessageContainer = function() {
        this.questionButtonMessage = $('<div>').attr({
          "class": 'question-button-message visually-hidden',
          id: "question-button-message-" + this.id
        });
        return this.buttonBay.append(this.questionButtonMessage);
      };

      /*
      
      updateQuestionButtonMessage(e)
      -------------
      Updates accessibility message container with specific message depending on which button was clicked.
      Once button has been clicked the message gets read by screen readers.
      */


      Question.prototype.updateQuestionButtonMessage = function(e) {
        var button,
          _this = this;
        button = Object.values(this.buttons).find(function(item) {
          return item.el.attr('id') === e.currentTarget.id;
        });
        if (button && button.statusMessage) {
          return this.questionButtonMessage.html($('<div>').attr('role', 'alert').text(button.statusMessage));
        }
      };

      return Question;

    })(TeachingElement);
  });

}).call(this);
