define(function(require) {
  const $ = require('jquery');
  const _ = require('underscore');
  require('backbone');
  const BackboneRadio = require('backbone.radio');
  const reconnectingWebsocket = require('common/js/vendor/reconnecting-websocket');
  const Student = require('courseware/js/app/models/Student');
  const AnnouncementNotifications = require('courseware/js/app/models/AnnouncementNotifications');
  // TODO - convert this file to ES6 imports - then .default won't be necessary
  const ActivityTracker = require('dist/js/common/user-session-timeout.bundle.min').default;
  require('bootstrap');
  require('dist/js/common/ClientsideLogger.bundle.min');
  require('common/js/app/event_metric');

  var _this = window;
  var notificationChannel = BackboneRadio.channel('notifications');
  var metricsChannel = BackboneRadio.channel('metrics');
  var disabledComponentsChannel = BackboneRadio.channel('disabledComponents');

  // This has to be synced with backend
  const NOTIFICATION = {
    TYPE: {
      ANNOUNCEMENT: 'announcement',
      DISABLED_COMPONENTS: 'disabledComponents'
    },
    PAYLOAD: {
      DISABLED_COMPONENTS: 'disabled_components_list',
    }
  };

  // TBN data send request.
  notificationChannel.on('tbn_send', function(notification) {
    _this.hbx_socket.send(JSON.stringify(notification));
  });

  // Consensus data send request.
  notificationChannel.on('consensus_send', function(notification) {
    _this.hbx_socket.send(JSON.stringify(notification));
  });

  // Navigation metrics data send request.
  metricsChannel.on('event_metric_package', function(metricPackage) {
    _this.hbx_socket.send(JSON.stringify(metricPackage));
  });

  // Manage HBX notification service
  function handleRuntimeNotification() {
    _this.hbx_socket = new reconnectingWebsocket(window.globals.notificationServiceHost);

    _this.hbx_socket.onmessage = function(message) {
      processRuntimeNotificationMessage(message)
    };

    _this.hbx_socket.onclose = function() {
      _this.hbx_socket.send(JSON.stringify({'closed': 'closed'}));
    };
  }

  var processRuntimeNotificationMessage = function(message) {
    const notificationMessage = $.parseJSON(message.data);
    if (notificationMessage.error) {
      throw notificationMessage.error;
    }
    if (notificationMessage.type === NOTIFICATION.TYPE.ANNOUNCEMENT) {
      ActivityTracker.addEmergencyAnnouncements([notificationMessage.data]);
      refreshAnnouncementBell(true);
    }
    if (notificationMessage.type === NOTIFICATION.TYPE.DISABLED_COMPONENTS) {
      disabledComponentsChannel.trigger(
        'disabledComponentsUpdated',
        notificationMessage.data[NOTIFICATION.PAYLOAD.DISABLED_COMPONENTS]
      );
    }
    // Handle sidebar notification
    notificationChannel.trigger(notificationMessage.type, notificationMessage);
  };

  if (_this.globals.sseStatus)
    handleRuntimeNotification();

  // Restricting announcements to Students
  if (window.user_type === 'student') {
    window.refreshAnnouncementBell = function(pushNotificationFlag) {
      if (pushNotificationFlag) {
        // Increment announcement counter on bell icon just in case if push notification of announcement type arrives
        updateAnnouncementCounterOnBellIcon();
      } else {
        fetchAnnouncementCounterAndUpdateBellIcon();
      }
    };

    var updateAnnouncementCounterOnBellIcon = function() {
      var $bellCounter = $('.nav-notf-counter');
      var count = parseInt($bellCounter.text()) + 1;
      setCounterAndStyleOnBellIcon(count);
    };

    var fetchAnnouncementCounterAndUpdateBellIcon = function() {
      var announcementNotifications = new AnnouncementNotifications();
      announcementNotifications.fetch({
        success: function() {
          ActivityTracker.addEmergencyAnnouncements(announcementNotifications.get('emergency_announcements'));
          setCounterAndStyleOnBellIcon(announcementNotifications.get('unread_counter'));
        }
      });
    };

    var setCounterAndStyleOnBellIcon = function(count) {

      var $bellCounter = $('.nav-notf-counter');
      if (count != 0) {
        $('.nav-notf').removeClass('nav-notf').addClass('nav-notf-active');
        $bellCounter.text(count).show();
        $('#show_announcements').attr('aria-valuetext', count + ' notifications').attr('aria-label', count + ' notifications');
      } else {
        $('.nav-notf-active').removeClass('nav-notf-active').addClass('nav-notf');
        $bellCounter.text(0).hide();
      }
    };

    function setFocus(evt) {
      var activityFeed = $('.activity-feed-all');
      var viewProfile = $('.view-my-profile');

      if (activityFeed.is(':visible')) {
        evt.preventDefault();
        activityFeed.focus();
      } else if (viewProfile.is(':visible')) {
        evt.preventDefault();
        viewProfile.focus();
      }
    }

    $('#account_dropdown').on('keydown', function(evt) {
      var dropdown_item = $('.nav-password');
      if (evt.keyCode == 9 && !event.shiftKey && dropdown_item.is(':visible') == false) {
        setFocus(evt);
      } else if (evt.keyCode == 13) {
        $(evt.target).trigger('click');
      }

    });

    $('#my_courses_dropdown, #show_announcements, #emotions_link, li').on('keyup', function(evt) {
      if (evt.keyCode == 13) {
        if (evt.target.id == 'my_courses')
          $('#' + evt.target.id)[0].click();
        else
          $(evt.target).trigger('click');
      }
    });

    var thisStudent = new Student({id: window.user_id});

    $('#emotions_dropdown li a').on('click', function(event) {
      event.preventDefault();
    });
    $('#emotions_dropdown li').on('click', function() {
      var element = this;
      thisStudent.save({
          feeling: $(element).text(),
          purify: $(element).text()
        },
        {
          patch: true,
          success: function() {
            $('#icon_emotions_active').attr('class', $(element).find('i').attr('class'));
            var emotionsTooltip = $('#icon_emotions_active').parent().parent();
            emotionsTooltip.attr('data-original-title', (thisStudent.get('feeling') || 'EMOTIONS').toUpperCase());
            emotionsTooltip.attr('aria-label', (thisStudent.get('feeling') || 'EMOTIONS').toUpperCase());

            if ($('#profile_icons').length > 0) {
              window.current_user_profile.model.fetch();
              window.current_user_profile.delegateEvents();
            }
          }
        });
    });

    $('.dropdown-toggle').on('click', function(event) {
      var dropdown = $(event.target);
      if (!dropdown.hasClass('open')) {

        var dropdownMenu = dropdown.next('.dropdown-menu');
        var firstMenuItem = dropdownMenu.find('li').first();
        var lastMenuItem = dropdownMenu.find('li').last();
        // set focus on first input
        firstMenuItem.focus();

        // redirect last tab to first input
        lastMenuItem.on('keydown', function(e) {
          if ((e.which === 9 && !e.shiftKey)) {
            e.preventDefault();
            firstMenuItem.focus();
          }
        });

        // redirect first shift+tab to last input
        firstMenuItem.on('keydown', function(e) {
          if ((e.which === 9 && e.shiftKey)) {
            e.preventDefault();
            lastMenuItem.focus();
          }
        });

      }
    });

    window.refreshAnnouncementBell(false);
  }
});
