// Generated by CoffeeScript 1.6.3
/*
Tooltip
=====
Provides generic tooltip functions
*/


(function() {
  define(function(require) {
    var $, Tooltip, defaults;
    $ = require('jquery');
    /*
    Configuration
    =============
    The tooltip utility allows for customizing some aspects of the tooltip
    */

    defaults = {
      /*
      *   placement: where the tooltip will appear
      */

      placement: "top",
      /*
      *   trigger: how the tooltip displays
      */

      trigger: "hover focus manual",
      /*
      *   container: Appends the tooltip to a specific element
      */

      container: false,
      /*
      *   html: determines if the tooltip will display html content
      */

      html: true
    };
    Tooltip = (function() {
      function Tooltip() {
        this.tmpl = "<div class=\"tooltip\" role=\"tooltip\" id=\"__id__\">\n  <div class=\"tooltip-arrow\"></div>\n  <div class=\"tooltip-inner\"></div>\n</div>";
      }

      /*
      buildTooltip()
      ------
      Creates tooltip for the element
      */


      Tooltip.prototype.buildTooltip = function($target, id, content, options, includeAriaDescribedby) {
        var config;
        if (options == null) {
          options = {};
        }
        if (includeAriaDescribedby == null) {
          includeAriaDescribedby = true;
        }
        config = $.extend({}, defaults, options);
        $target.attr("tabindex", 0);
        if (includeAriaDescribedby && $target.attr("aria-describedby") !== id) {
          $target.attr("aria-describedby", id);
        }
        return $target.tooltip("destroy").tooltip({
          html: config.html,
          placement: config.placement,
          title: content,
          trigger: config.trigger,
          template: this.tmpl.replace("__id__", id),
          container: config.container
        });
      };

      /*
      showTooltip()
      ------
      Displays the tooltip for the element
      */


      Tooltip.prototype.showTooltip = function($target) {
        return $target.tooltip("show");
      };

      /*
      updateTitle()
      ------
      Update title of tooltip
      */


      Tooltip.prototype.updateTitle = function($target, title, show) {
        if (show == null) {
          show = true;
        }
        $target.attr("data-original-title", title).tooltip("fixTitle");
        if (show) {
          return this.showTooltip($target);
        }
      };

      /*
      hideTooltip()
      ------
      Destroys the tooltip for the element
      */


      Tooltip.prototype.hideTooltip = function($target) {
        return $target.tooltip("hide");
      };

      /*
      destroyTooltip()
      ------
      Destroys the tooltip for the element
      */


      Tooltip.prototype.destroyTooltip = function($target, includeAriaDescribedby) {
        if (includeAriaDescribedby == null) {
          includeAriaDescribedby = true;
        }
        $target.tooltip("destroy");
        if (!includeAriaDescribedby && $target.attr("aria-describedby")) {
          return $target.removeAttr("aria-describedby");
        }
      };

      /*
      getTooltipPlacement()
      ------
      Get placement for tooltips that don't specify their location
      */


      Tooltip.prototype.getTooltipPlacement = function(defaultPosition, width, height) {
        var placement;
        placement = function(tooltip, element) {
          var $element, boundLeft, boundRight, isWithinBounds, pos, testLeft, testRight;
          $element = $(element);
          boundLeft = $(document).scrollLeft();
          boundRight = (boundLeft + $(window).width()) / 2 + 700;
          pos = $.extend(true, {}, $element.offset(), {
            width: element.offsetWidth,
            height: element.offsetHeight
          });
          isWithinBounds = function(elPos) {
            return boundLeft < elPos.left && boundRight > (elPos.left + width);
          };
          testLeft = function() {
            return isWithinBounds({
              top: pos.top + pos.height / 2 - height / 2,
              left: pos.left - width
            });
          };
          testRight = function() {
            return isWithinBounds({
              top: pos.top + pos.height / 2 - height / 2,
              left: pos.left + width
            });
          };
          if (testRight()) {
            return "right";
          }
          if (testLeft()) {
            return "left";
          }
          return defaultPosition;
        };
        return placement;
      };

      return Tooltip;

    })();
    return new Tooltip;
  });

}).call(this);
