// Generated by CoffeeScript 1.6.3
/*
Utils
=====
Provides generic utility functions as a singleton object.
*/


(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  define(function(require) {
    var $, Utils;
    $ = require('jquery');
    Utils = (function() {
      function Utils() {
        this.isTypeOfKey = __bind(this.isTypeOfKey, this);
        this.strToCamelCase = __bind(this.strToCamelCase, this);
        this.ucwords = __bind(this.ucwords, this);
        this.normalizeParticipantContextURL = __bind(this.normalizeParticipantContextURL, this);
        this.normalizeUrl = __bind(this.normalizeUrl, this);
        this.apiURL = '/api/v1/tep';
        this.participantContextURL = '/participant_context/';
        this.harness = false;
        this.stases = ['locked', 'unlocked', 'in-progress', 'complete'];
      }

      /*
      as_decimal(raw, precision)
      --------------------------
      Given a float raw value and an integer precision, returns a string
      representation of the float to the specified number of decimal places,
      zero-padding as necessary.
      */


      Utils.prototype.as_decimal = function(raw, precision) {
        var multiplier, result;
        if (precision == null) {
          precision = 2;
        }
        multiplier = Math.pow(10, precision);
        result = Math.round(multiplier * raw) / multiplier + "";
        return this._check_zero_pad(result, precision);
      };

      /*
      _check_zero_pad(result, precision)
      -------------------------------
      Internal method to make sure we have the right number of zeroes on integers that
      should look like floats. This seems a little torturous, but the float -> string conversion
      swallows the precision padding if the number happens to not have value in the decimal places
      */


      Utils.prototype._check_zero_pad = function(result, precision) {
        var missing_zeroes, pieces;
        pieces = result.split(".");
        if (!pieces) {
          return result;
        }
        if (pieces.length === 1) {
          return pieces[0] + "." + this.zero_pad(precision);
        }
        missing_zeroes = precision - pieces[1].length;
        if (missing_zeroes === 0) {
          return result;
        }
        return ("" + pieces[0] + "." + pieces[1]) + this.zero_pad(missing_zeroes);
      };

      /*
      
      locale_formatted_decimal(raw, precision)
      ---------------
      Apply as_decimal above and then format with commas
      */


      Utils.prototype.locale_formatted_decimal = function(raw, precision) {
        return this._check_zero_pad(this.commaNumber(parseFloat(this.as_decimal(raw, precision))), precision);
      };

      /*
      
      zero_pad(count)
      ---------------
      Returns a string of "0" of length count.
      */


      Utils.prototype.zero_pad = function(count) {
        return Array(count + 1).join("0");
      };

      /*
      isGUID(str)
      -----------
      Returns whether the provided string is a valid MongoID.
      
      Note that this method only checks that it is a 24-digit hexidecimal string.
      */


      Utils.prototype.isGUID = function(str) {
        var m, _ref;
        m = str.match(/^[a-f\d]{24}$/gi);
        return (_ref = m != null) != null ? _ref : m.length > {
          0: false
        };
      };

      /*
      normalizeUrl(url, method = "GET")
      ---------------------------------
      Wraps a url string with the appropriate pre/suffix, based on the provided method.
      
      This allows for easier, and more easily configurable, API calls.
      */


      Utils.prototype.normalizeUrl = function(url, method) {
        if (method == null) {
          method = "GET";
        }
        switch (method) {
          case "GET":
            return this.apiURL + url + (this.harness ? ".json" : "?format=json&course_instance_id=" + window.globals.courseInstanceId + "&cohort_id=" + window.cohort_id);
          case "POST":
            return this.apiURL + url;
        }
      };

      Utils.prototype.normalizeParticipantContextURL = function(url, method) {
        if (method == null) {
          method = "GET";
        }
        switch (method) {
          case "GET":
            return this.participantContextURL + url + "?format=json&course_instance_id=" + window.globals.courseInstanceId + "&cohort_id=" + window.cohort_id;
          case "POST":
            return this.participantContextURL + url;
        }
      };

      /*
      measureText(text, font)
      -----------------------
      Given a string and a css font descriptor, provides an approximate length of
      the string in pixels.
      */


      Utils.prototype.measureText = function(text, font) {
        var s, width;
        s = $('<span>').text(text).css({
          'font': font,
          'white-space': 'nowrap',
          'visibility': 'hidden'
        });
        $('body').append(s);
        width = s.width();
        s.remove();
        return width;
      };

      /*
      normalizeEventPosition(jqevt)
      -----------------------------
      Given a jQuery mouse or touch event, returns the position of the event relative to the page.
      If multitouch, only the first touch is evaluated.
      
      This method is depreciated in favor of the MouseTouch utility.
      */


      Utils.prototype.normalizeEventPosition = function(jqevt) {
        var target, _ref;
        target = jqevt;
        if (((_ref = jqevt.type) === 'touchstart' || _ref === 'touchmove') && jqevt.touches.length > 0) {
          target = jqevt.touches[0];
        }
        return {
          x: target.pageX,
          y: target.pageY
        };
      };

      /*
      timeToString(time)
      ------------------
      Given a time in milliseconds, returns a H:m:s formatted string.
      */


      Utils.prototype.timeToString = function(time) {
        var h, hlen, m, mlen, s, slen;
        hlen = 60 * 60 * 1000;
        mlen = 60 * 1000;
        slen = 1000;
        h = Math.floor(time / hlen);
        m = Math.floor((time % hlen) / mlen);
        s = Math.floor((time % mlen) / slen);
        if (s < 10) {
          s = '0' + s;
        }
        if (m < 10 && h) {
          m = '0' + m;
        }
        return (h ? h + ":" : "") + (h || m ? m + ":" : "0:") + s;
      };

      /*
      getCurrentTime()
      ------------------
      Get the current time in a user friendly string
      */


      Utils.prototype.getCurrentTime = function(time) {
        var current_time, dd, h, m, mm, months, norm_hour, period, yyyy;
        period = function() {
          if (h > 11) {
            return "PM";
          } else {
            return "AM";
          }
        };
        norm_hour = function() {
          var hour;
          if (h === 12) {
            return 12;
          } else if (h === 0) {
            return 12;
          } else {
            hour = h % 12;
            return hour;
          }
        };
        if (time != null) {
          current_time = new Date(0);
          current_time.setUTCSeconds(time);
        } else {
          current_time = new Date();
        }
        dd = current_time.getUTCDate();
        months = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
        mm = months[current_time.getUTCMonth()];
        yyyy = current_time.getUTCFullYear();
        h = current_time.getUTCHours();
        m = current_time.getUTCMinutes();
        if (dd < 10) {
          dd = "0" + dd;
        }
        if (m < 10) {
          m = "0" + m;
        }
        return current_time = "" + mm + " " + dd + ", " + yyyy + " " + (norm_hour()) + ":" + m + " " + (period());
      };

      /*
      String Utilities
      ================
      
      ucfirst(str)
      ------------
      Returns a copy of the provided string with the first letter capitalized.
      */


      Utils.prototype.ucfirst = function(str) {
        var f;
        f = str.charAt(0).toUpperCase();
        return f + str.substr(1);
      };

      /*
      
      ucwords(str)
      ------------
      Returns a copy of the string with each word capitalized.
      */


      Utils.prototype.ucwords = function(str) {
        return this.ucfirst(str.replace(/(\s)(\w)/, function(match, p1, p2, offset, string) {
          return p1 + p2.toUpperCase();
        }));
      };

      /*
      
      strToCamelCase(str)
      -------------------
      Returns a copy of the provided string, camelized (spaces removed, first letter
      of each word capitalized).
      */


      Utils.prototype.strToCamelCase = function(str) {
        var despace, word, _i, _len;
        despace = str.split("\s");
        for (_i = 0, _len = despace.length; _i < _len; _i++) {
          word = despace[_i];
          this.ucfirst(word);
        }
        return despace.join('');
      };

      /*
      
      camelToSpace(camel)
      -------------------
      Returns a decamelized copy of the provided camelCase string.
      */


      Utils.prototype.camelToSpace = function(camel) {
        var str;
        return str = camel.replace(/([A-Z])/, function(match, p1) {
          return ' ' + p1.toLowerCase();
        });
      };

      /*
      
      strToSnakeCase(str)
      -------------------
      Returns a copy of the provided string, snaked (all lower case, spaces replaced
      with underscores).
      */


      Utils.prototype.strToSnakeCase = function(str) {
        var despace;
        despace = str.toLowerCase().split("\s");
        return despace.join('_');
      };

      /*
      commaNumber(num)
      ----------------
      Returns the number with comma separators (e.g. 1,000 instead of 1000).
      
      The use of commas is culture/language dependent, so we defer to toLocaleString,
      though if this is insufficient we can easily override it here.
      */


      Utils.prototype.commaNumber = function(num) {
        if ((num != null) && isNaN(num)) {
          num = this.stripCommas(num);
        }
        if (isNaN(num)) {
          return '';
        } else {
          return num.toString().replace(/\B(?=(?:\d{3})+(?!\d))/g, ",");
        }
      };

      /*
      stripCommas(num)
      ----------------
      Returns a value with commas removed
      */


      Utils.prototype.stripCommas = function(num) {
        return num.replace(/\,/g, "");
      };

      /*
      undecorateValue(val)
      ----------------
      Removes non number characters from a string
      */


      Utils.prototype.undecorateValue = function(val) {
        var strip_val;
        if (val) {
          strip_val = val.replace(/[^0-9\.]+/g, "");
          if (!isNaN(strip_val)) {
            if (strip_val) {
              return Number(strip_val);
            } else {
              return val;
            }
          }
        }
        return false;
      };

      /*
      hyphenateWords(words, charLimit)
      --------------------------------
      Given a string, returns that same string with hypens and <br/>s inserted, based
      on the provided line length charLimit.
      */


      Utils.prototype.hyphenateWords = function(words, charLimit) {
        var cnt, l, letters, newLetters, newWord, pos, w, wcnt, _i, _j, _len, _len1;
        words = words.split(" ");
        wcnt = 0;
        for (_i = 0, _len = words.length; _i < _len; _i++) {
          w = words[_i];
          if (w.length > charLimit) {
            letters = w.split("");
            newLetters = '';
            cnt = 1;
            pos = 0;
            for (_j = 0, _len1 = letters.length; _j < _len1; _j++) {
              l = letters[_j];
              if (cnt === charLimit && l !== " " && letters[pos + 1] !== void 0) {
                l += '-<br />';
                cnt = 0;
              }
              newLetters += l;
              cnt++;
              pos++;
            }
            newWord = newLetters;
            words[wcnt] = newWord;
          }
          wcnt++;
        }
        return words = words.join(" ");
      };

      Utils.prototype.countInstances = function(haystack, needle) {
        var inst, reg;
        reg = RegExp('\\b' + needle + '\\b', 'ig');
        inst = haystack.match(reg);
        if (inst != null) {
          return inst.length;
        } else {
          return 0;
        }
      };

      Utils.prototype.startsWith = function(word, prefix) {
        if (typeof word === "string" && (word != null)) {
          return word.indexOf(prefix) === 0;
        }
        return false;
      };

      Utils.prototype.endsWidth = function(word, suffix) {
        if (typeof word === "string" && (word != null)) {
          return word.indexOf(suffix, word.length - suffix.length) !== -1;
        }
        return false;
      };

      Utils.prototype.trim = function(text) {
        if (typeof word === "string" && (text != null)) {
          return text.replace(/^\s+|\s+$/g, '');
        }
        return text;
      };

      Utils.prototype.encodeHtmlCharacters = function(txt) {
        if ((txt != null) && txt !== "" && txt !== 'undefined') {
          return txt.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
        } else {
          return "";
        }
      };

      Utils.prototype.decodeHtmlCharacters = function(txt) {
        if ((txt != null) && txt !== "" && txt !== 'undefined') {
          return txt.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"');
        } else {
          return "";
        }
      };

      Utils.prototype.stripText = function(txt) {
        if ((txt != null) && txt !== "") {
          return txt.replace(/(<([^>]+)>)/ig, '');
        } else {
          return txt;
        }
      };

      /*
      
      decodeEntities(text)
      --------------------------------
      Unescape HTML entities in string
      */


      Utils.prototype.decodeEntities = function(text) {
        return $('<textarea/>').html(text).text();
        /*
        Audio Utilities
        ===============
        
        loadSound(file)
        ---------------
        Provided a file name without extension, creates and returns a new Audio object.
        Loads an mp3 or wav file, based on browser capabilities.
        */

      };

      Utils.prototype.loadSound = function(fileNoExt) {
        var canPlayMP3, snd;
        snd = new Audio();
        canPlayMP3 = snd.canPlayType && snd.canPlayType('audio/mpeg');
        if (canPlayMP3 !== '') {
          snd = new Audio(fileNoExt + '.mp3');
        } else {
          snd = new Audio(fileNoExt + '.wav');
        }
        return snd;
      };

      /*
      
      resetSound(snd)
      ---------------
      Sets the provided audio object's playhead to 0 and starts over.
      */


      Utils.prototype.resetSound = function(snd) {
        snd.currentTime = 0;
        return snd.play();
      };

      /*
      
      playSound(snd)
      --------------
      Begins playing the provided audio object.
      */


      Utils.prototype.playSound = function(snd) {
        return snd.play();
      };

      /*
      
      pauseSound(snd)
      --------------
      Pauses the playback of the provided audio object.
      */


      Utils.prototype.pauseSound = function(snd) {
        return snd.pause();
      };

      /*
      
      generateGUID4()
      ---------------
      Generates and returns a random version 4 GUID
      */


      Utils.prototype.generateGUID4 = function() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
          var r, v;
          r = Math.floor(Math.random() * 16);
          v = c === 'x' ? r : r & 0x3 | 0x8;
          return v.toString(16);
        });
      };

      /*
      clearSelection()
      ----------------
      Clears any selection the user may have made when clicking & dragging.
      */


      Utils.prototype.clearSelection = function() {
        if (window.getSelection != null) {
          if (window.getSelection().empty != null) {
            return window.getSelection().empty();
          } else if (window.getSelection().removeAllRanges != null) {
            return window.getSelection().removeAllRanges();
          }
        } else if ((document.selection != null) && (document.selection.empty != null)) {
          return document.selection.empty();
        }
      };

      /*
      getKeyMap()
      ------
      Get common keys used for interaction; used mostly to support keyboard only controls
      */


      Utils.prototype.getKeyMap = function() {
        var downKey, endKey, enterKey, escapeKey, homeKey, leftKey, pgDown, pgUp, rightKey, spaceKey, upKey;
        escapeKey = 27;
        enterKey = 13;
        spaceKey = 32;
        pgUp = 33;
        pgDown = 34;
        endKey = 35;
        homeKey = 36;
        leftKey = 37;
        upKey = 38;
        rightKey = 39;
        downKey = 40;
        return {
          allKeys: [escapeKey, enterKey, spaceKey, pgUp, pgDown, endKey, homeKey, leftKey, upKey, rightKey, downKey],
          allMovementKeys: [pgUp, pgDown, endKey, homeKey, leftKey, upKey, downKey, rightKey],
          increaseKeys: [upKey, rightKey, pgUp],
          decreaseKeys: [downKey, leftKey, pgDown],
          jumpKeys: [pgUp, pgDown],
          zeroKey: homeKey,
          maxKey: endKey,
          selectKey: spaceKey,
          submitKey: enterKey,
          escapeKey: escapeKey
        };
      };

      /*
      isTypeOfKey()
      ------
      Send in an event to test if the key code matches the supplied type
      Keys can belong to separate groups so the type needs be specified
      */


      Utils.prototype.isTypeOfKey = function(type, evt) {
        var code, key, keys;
        code = evt.keyCode ? evt.keyCode : evt.which;
        keys = this.getKeyMap();
        if (keys[type] != null) {
          key = keys[type];
          if (typeof key === "object") {
            if (__indexOf.call(key, code) >= 0) {
              return true;
            } else {
              return false;
            }
          } else {
            if (code === key) {
              return true;
            } else {
              return false;
            }
          }
        } else {
          return false;
        }
      };

      return Utils;

    })();
    return new Utils;
  });

}).call(this);
